CREATE TABLE npcDiscount (
  discountID INT(11) NOT NULL AUTO_INCREMENT,
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  description TEXT,
  isActive TINYINT(4) NOT NULL DEFAULT '1',
  usersAppliesType VARCHAR(32) DEFAULT NULL,
  usersAppliesValue VARCHAR(255) DEFAULT NULL,
  itemsAppliesType VARCHAR(32) DEFAULT NULL,
  itemsAppliesValue VARCHAR(255) DEFAULT NULL,
  rewardType VARCHAR(32) DEFAULT NULL,
  reward DECIMAL(10,4) NULL,
  couponCode VARCHAR(100) DEFAULT NULL,
  startDate VARCHAR(100) DEFAULT NULL,
  expirationDate VARCHAR(100) DEFAULT NULL,
  timesUsed INT(11) DEFAULT NULL,
  maxTimesUsed INT(11) DEFAULT NULL,
  isTimesPerCustomer TINYINT(4) NOT NULL DEFAULT '1',
  discountOrder INT(11) NOT NULL DEFAULT '0',
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.DiscountBean',
  PRIMARY KEY  (discountID),
  UNIQUE KEY discountCodeUKey (code)
);

CREATE INDEX IDX_npcDiscount_name ON npcDiscount (name);
CREATE INDEX IDX_npcDiscount_discountOrder ON npcDiscount (discountOrder);

CREATE TABLE npcSKUDiscount (
  skuDiscountID INT(11) NOT NULL AUTO_INCREMENT,
  skuID INT(11) NOT NULL DEFAULT '0',
  discountID INT(11) NOT NULL DEFAULT '0',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.SKUDiscountBean',
  PRIMARY KEY  (skuDiscountID),
  UNIQUE KEY discountCodeUKey (skuID, discountID),
  KEY skuIDIndex (skuID),
  FOREIGN KEY (`skuID`) REFERENCES `npcSKU` (`skuID`) ON DELETE CASCADE,
  KEY discountIDIndex (discountID),
  FOREIGN KEY (`discountID`) REFERENCES `npcDiscount` (`discountID`) ON DELETE CASCADE
);


CREATE TABLE npcDiscountRange (
  discountRangeID INT(11) NOT NULL AUTO_INCREMENT,
  discountID INT(11) NOT NULL,
  floor DECIMAL(10,4) NOT NULL DEFAULT '0',
  ceiling DECIMAL(10,4) NOT NULL DEFAULT '0',
  rewardType VARCHAR(32) DEFAULT NULL,
  reward DECIMAL(10,4) NOT NULL DEFAULT '0',
  isActive TINYINT(4) NOT NULL DEFAULT '1',
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.DiscountRangeBean',
  PRIMARY KEY  (discountRangeID),
  KEY discountIDIndex (discountID),
  FOREIGN KEY (`discountID`) REFERENCES `npcDiscount` (`discountID`) ON DELETE CASCADE
);

CREATE INDEX IDX_npcDiscountRange_reward ON npcDiscountRange (reward);

CREATE TABLE npcRelatedProduct (
  relatedProductID INT(11) NOT NULL AUTO_INCREMENT,
  parentProductID INT(11) NOT NULL DEFAULT '0',
  productID INT(11) NOT NULL DEFAULT '0',
  relatedType VARCHAR(32) DEFAULT NULL,
  relatedProductOrder INT(11) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.product.RelatedProductBean',
  PRIMARY KEY  (relatedProductID),
  UNIQUE KEY discountCodeUKey (parentProductID, productID),
  KEY productIDIndex (productID),
  FOREIGN KEY (`productID`) REFERENCES `npcProduct` (`productID`)
);

CREATE TABLE npcOrderDiscount (  
  orderDiscountID INT(11) NOT NULL AUTO_INCREMENT,
  orderID INT(11) NOT NULL DEFAULT '0',
  orderItemID INT(11) NULL,
  total DECIMAL(10,4) DEFAULT NULL,
  discountID INT(11) NOT NULL DEFAULT '0',
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  usersAppliesType VARCHAR(32) DEFAULT NULL,
  usersAppliesValue VARCHAR(255) DEFAULT NULL,
  itemsAppliesType VARCHAR(32) DEFAULT NULL,
  itemsAppliesValue VARCHAR(255) DEFAULT NULL,
  rewardType VARCHAR(32) DEFAULT NULL,
  reward DECIMAL(10,4) NOT NULL DEFAULT '0',
  couponCode VARCHAR(100) DEFAULT NULL,
  floor DECIMAL(10,4) NOT NULL DEFAULT '0',
  ceiling DECIMAL(10,4) NOT NULL DEFAULT '0',
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderDiscountBean',
  PRIMARY KEY  (orderDiscountID),
  KEY orderIDIndex (orderID),
  FOREIGN KEY (`orderID`) REFERENCES `npcOrder` (`orderID`) ON DELETE CASCADE
);

CREATE INDEX IDX_npcOrderDiscount_total ON npcOrderDiscount (total);
CREATE INDEX IDX_npcOrderDiscount_code ON npcOrderDiscount (code);
CREATE INDEX IDX_npcOrderDiscount_name ON npcOrderDiscount (name);
