package com.softslate.commerce.administrator.core;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.AppMessageResources;
import com.softslate.commerce.customer.core.AppUtils;

/**
 * @author David Tobey
 * 
 */
public class StoreMessagesAction extends SettingsAction {

	static Log log = LogFactory.getLog(StoreMessagesAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		StoreMessagesForm baseForm = (StoreMessagesForm) form;
		baseForm.initializeProperties(mapping, request);
		String keyPrefix = ".";
		synchronized (getServlet().getServletContext()) {

			// Keep track of default, custom, and combined components
			Properties nullProps = null;
			String defaultPropertiesFileName = "/resources/"
					+ baseForm.getBundle() + "/application.properties";
			Properties defaultProperties = AppUtils.loadProperties(
					defaultPropertiesFileName, nullProps);
			String customPropertiesFileName = getServlet().getServletContext()
					.getRealPath(
							"/WEB-INF/classes/resources/"
									+ baseForm.getBundle()
									+ "/application-custom.properties");
			File customPropertiesFile = new File(customPropertiesFileName);
			Properties customProperties = AppUtils
					.loadProperties(customPropertiesFile);
			if (log.isInfoEnabled())
				log.info("customProperties: " + customProperties);

			// Find the messages Map we need to alter
			AppMessageResources coreResources = (AppMessageResources) getServlet()
					.getServletContext().getAttribute(Globals.MESSAGES_KEY);
			Map customCoreMessages = coreResources.getCustomMessageResources()
					.getMessages();
			Map coreMessages = coreResources.getMessages();
			if (log.isDebugEnabled())
				log.debug("customCoreMessages: " + customCoreMessages);
			if (log.isDebugEnabled())
				log.debug("coreMessages: " + coreMessages);
			AppMessageResources resources = (AppMessageResources) getServlet()
					.getServletContext().getAttribute(
							"/" + baseForm.getBundle());
			Map customMessages = resources.getCustomMessageResources()
					.getMessages();
			Map messages = resources.getMessages();

			// Only save properties whose messages are found and not the default
			ActionUtils actionUtils = (ActionUtils) baseForm
					.getBusinessObjectFactory().createUtilsObject(
							"actionUtilsImplementer");
			Map parameters = actionUtils.buildParameterMap(request);
			Set keys = parameters.keySet();
			Iterator i = keys.iterator();
			while (i.hasNext()) {
				String fullKey = (String) i.next();
				if (log.isDebugEnabled())
					log.debug("Full Key: " + fullKey);

				// Skip known request parameters that aren't messages
				if (fullKey.equals("bundle")
						|| fullKey.equals("storeMessagesSubmit")
						|| fullKey.equals("searchString")) {
					continue;
				}

				String value = (String) parameters.get(fullKey);
				String key = fullKey.replaceFirst(keyPrefix, "");

				// Remove if the new value is the same as the default
				if (value.equals(defaultProperties.getProperty(key))) {

					if (log.isInfoEnabled())
						log.info("Removing message - same as the default: "
								+ key);
					customProperties.remove(key);
				} else {

					// Save in the properties object that will be saved to the
					// file later.
					customProperties.setProperty(key, value);

				}

				if (log.isInfoEnabled())
					log.info("Saving message: " + key);

				// Save in memory for all locales. (This tool does not do
				// translations.)
				saveMessageInMemory(customMessages, key, value);
				saveMessageInMemory(messages, key, value);
				if (baseForm.getBundle().equals("core")) {
					saveMessageInMemory(customCoreMessages, key, value);
					saveMessageInMemory(coreMessages, key, value);
				}
			}

			// Save the custom components
			saveProperties(customProperties, baseForm);
			resources.getFormats().clear();
			resources.getCustomMessageResources().getFormats().clear();
			if (baseForm.getBundle().equals("core")) {
				coreResources.getFormats().clear();
				coreResources.getCustomMessageResources().getFormats().clear();
			}

		}
		baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
				new ActionMessage("messages.settingsMessage"));
		if (!baseForm.getMessages().isEmpty())
			saveMessages(request, baseForm.getMessages());
		return mapping.findForward("success");
	}

	/**
	 * Saves a single message in memory for a given Map of messages, across all
	 * locales.
	 * 
	 * @param messages
	 * @param key
	 * @param value
	 */
	public void saveMessageInMemory(Map messages, String key, String value) {

		// Save in memory for all locales. (This tool does do
		// translations.)
		synchronized (messages) {
			Iterator j = messages.keySet().iterator();
			while (j.hasNext()) {
				String fullKey = (String) j.next();

				// Compare everything after the first . with the key, to
				// cover all locales
				int dotIndex = fullKey.indexOf(".");
				if (dotIndex != -1) {
					String lastPartOfFullKey = fullKey.substring(dotIndex + 1);
					if (lastPartOfFullKey.equals(key)) {

						// Update in memory
						if (log.isDebugEnabled())
							log.debug("Saving in memory: " + fullKey
									+ "value: " + value);
						messages.put(fullKey, value);
					}
				}
			}
		}

	}

	/**
	 * Saves a new set of properties for the give custom store messages.
	 * 
	 * @param customProperties
	 *            The new set of properties to be saved.
	 * @param baseForm
	 *            The StoreMessageForm instance being processed.
	 * @throws Exception
	 */
	public void saveProperties(Properties customProperties,
			StoreMessagesForm baseForm) throws Exception {
		String customPropertiesFileName = getServlet().getServletContext()
				.getRealPath(
						"/WEB-INF/classes/resources/" + baseForm.getBundle()
								+ "/application-custom.properties");
		File messagesFile = new File(customPropertiesFileName);
		messagesFile.createNewFile();
		FileOutputStream fos = new FileOutputStream(messagesFile);
		if (log.isDebugEnabled())
			log.debug("Attempting to write messages to "
					+ customPropertiesFileName + " custom messages: "
					+ customProperties);
		customProperties.store(fos, "Generated by Administrator.");

		// In case we're in an environment with an src folder, save
		// the file to the src area too.
		String customPropertiesFileNameSrc = null;
		try {
			customPropertiesFileNameSrc = getServlet().getServletContext()
					.getRealPath(
							"/WEB-INF/src/resources/" + baseForm.getBundle()
									+ "/application-custom.properties");
			File messagesFileSrc = new File(customPropertiesFileNameSrc);
			messagesFileSrc.createNewFile();
			FileOutputStream fosSrc = new FileOutputStream(messagesFileSrc);
			if (log.isDebugEnabled())
				log.debug("Attempting to write messages to "
						+ customPropertiesFileNameSrc + " custom messages: "
						+ customProperties);
			customProperties.store(fosSrc, "Generated by Administrator.");
		} catch (Exception e) {
			if (log.isInfoEnabled())
				log.info("Error attempting to write messages to "
						+ customPropertiesFileNameSrc, e);
		}
	}
}
