package com.softslate.commerce.administrator.core;

import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.ApplicationMessage;
import com.softslate.commerce.businessobjects.core.Setting;
import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/UserSettingAdd.do". Processes a request to add a record to
 * the system.
 * 
 * @author Jason McWilliams
 * 
 */
public class UserSettingAddAction extends BaseAction {

	static Log log = LogFactory.getLog(UserSettingAddAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		// Add the record
		BaseAdministratorForm baseForm = (BaseAdministratorForm) form;

		Settings settingsBean =  baseForm.getBusinessObjectFactory()
				.createObject(Settings.class);

		// Add setting
		Map results = settingsBean.addSetting(PropertyUtils.describe(form));

		// Reload all the settings
		Settings settings = (Settings) servlet.getServletContext()
				.getAttribute("settingsBean");
		settings.initialize();
		
		// Successful insert
		if (results == null || results.get("resultCode") == null || results.get("resultCode").equals("0")) {

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordAdded"));
			if (results != null && results.get("newRecord") != null) {
				Setting newRecord = (Setting) results.get("newRecord");

				// Set a request scope variable for sub classes to use
				request.setAttribute("newSettingID", new Integer(newRecord.getSettingID()));

				baseForm.getMessages().add( ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("messages.editNewRecord", "UserSettingEditForm.do?settingID="
								+ newRecord.getSettingID()));
				request.setAttribute("okNotToEscapeMessages", Boolean.TRUE);
			}

			// If successful, signal the form to be reinstantiated.
			request.setAttribute("reset", "true");
		}
		
		// Set the messages
		ActionUtils actionUtils = (ActionUtils) baseForm.getBusinessObjectFactory().createUtilsObject(
				"actionUtilsImplementer");
		actionUtils.processApplicationMessages(baseForm.getErrors(), baseForm.getMessages(),
				(Collection<ApplicationMessage>) results.get("applicationMessages"));
		saveErrors(baseForm.getRequest(), baseForm.getErrors());
		saveMessages(baseForm.getRequest(), baseForm.getMessages());

		return mapping.findForward("success");
	}
}
