package com.softslate.commerce.administrator.importexport;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.importexport.QboConnectionProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;
import com.softslate.commerce.daos.core.DAOFactory;

public class QboConnectAction extends BaseAction {

	static Log log = LogFactory.getLog(QboConnectAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		log.info("Starting action.");
		BaseForm baseForm = new BaseForm();
		baseForm.initializeProperties(mapping, request);
		BusinessObjectFactory bof = baseForm.getBusinessObjectFactory();
		QboConnectionProcessor qboProcessor = bof.createObject(QboConnectionProcessor.class);

		// Call the discovery API call to set various urls from QBO, which they could change on their end.
		// If the store switches from Sandbox to Live they'll need to connect again, and this will populate the
		// Live settings. Then they can switch back and forth at will because we'll have it all stored for them.
		// However, QBO recommends we update the settings every once in a while in case they change.
		Map results = qboProcessor.callDiscoveryDocument();
		if (results.get("error") != null) {
			saveErrorFromQboResult(baseForm, results);
			return mapping.findForward("failure");
		}

		// Save the settings with the global Settings object
		Settings settingsBean = bof.createObject(Settings.class);
		synchronized (getServlet().getServletContext()) {
			Settings settings = settingsBean.processSettings(results);
			servlet.getServletContext().setAttribute("settingsBean", settings);
			servlet.getServletContext().setAttribute("settings", settings);
			bof.setSettings(settings);
			request.setAttribute("settings", settings);
			DAOFactory daoFactory = (DAOFactory) getServlet().getServletContext().getAttribute("daoFactory");
			daoFactory.setSettings(settings);
			qboProcessor.setSettings(settings);
		}

		// Now try to connect. This returns a URL to QBO that we need to redirect the uesr to in their browser so they
		// can sign in and authorize the app to connect to their QBO company.
		String redirectURL = qboProcessor.prepareRedirectUrl((String) request.getSession().getAttribute("csrfToken"));
		log.info("Redirecting user to " + redirectURL);
		response.sendRedirect(redirectURL);
		return mapping.findForward("success");
	}

	private void saveErrorFromQboResult(BaseForm baseForm, Map results) {
		Map error = (Map) results.get("error");
		String resultCode = (String) error.get("resultCode");
		String resultMessage = (String) error.get("resultMessage");
		if (resultCode == null || !resultCode.equals("0")) {
			String message = baseForm.getMessage("importexport", "qbo.processingError" + resultCode, null);
			if (message == null || message.length() == 0) {
				if (resultMessage == null) {
					message = baseForm.getMessage("importexport", "qbo.processingError", null);
				} else {
					message = baseForm.getMessage("importexport", "qbo.processingErrorArg",
							new String[] { resultMessage });
				}
			}
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.detail", message));
			saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
			if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
				log.info("Validation error(s): " + baseForm.getErrors().toString());
		}
	}

}
