package com.softslate.commerce.administrator.order;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.order.Country;
import com.softslate.commerce.businessobjects.order.CountryProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/CountryEditForm.do". Loads in the current values of the
 * record being edited and populates a <code>CountryAddEditForm</code> object.
 * 
 * @author David Tobey
 * 
 */
public class CountryEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(CountryAddFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		CountryAddEditForm baseForm = (CountryAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		CountryProcessor processor = (CountryProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"countryProcessorImplementer");
		Country country = processor.loadCountryFromID(PropertyUtils
				.describe(form));
		BeanUtils.copyProperties(baseForm, country);

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("countryEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Countries",
				"Country Details" });
		request.setAttribute("breadcrumbs", bcs);

		request.setAttribute("country", country);
		request.setAttribute("countryAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
