package com.softslate.commerce.administrator.order;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.businessobjects.order.OrderItemAttribute;
import com.softslate.commerce.businessobjects.order.OrderItemAttributeProcessor;
import com.softslate.commerce.businessobjects.order.OrderItemProcessor;
import com.softslate.commerce.businessobjects.order.OrderProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/OrderItemAttributeEditForm.do". Loads in the current values
 * of the record being edited and populates a
 * <code>OrderItemAttributeAddEditForm</code> object.
 * 
 * @author David Tobey
 * 
 */
public class OrderItemAttributeEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(OrderItemAttributeAddFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		OrderItemAttributeAddEditForm baseForm = (OrderItemAttributeAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		OrderItemAttributeProcessor processor = (OrderItemAttributeProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"orderItemAttributeProcessorImplementer");
		OrderItemAttribute orderItemAttribute = processor
				.loadOrderItemAttributeFromID(PropertyUtils.describe(form));
		BeanUtils.copyProperties(baseForm, orderItemAttribute);
		request.setAttribute("orderItemAttribute", orderItemAttribute);

		// Load the parent order item
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		OrderItem orderItem = null;
		if (baseForm.getOrderItemID() != null
				&& baseForm.getOrderItemID().length() > 0) {
			OrderItemProcessor parentProcessor =  bof
					.createObject(OrderItemProcessor.class);
			orderItem = parentProcessor.loadOrderItemFromID(PropertyUtils
					.describe(baseForm));
			request.setAttribute("orderItem", orderItem);
		}

		// Load the parent order if an id is present
		Order order = null;
		if (orderItem != null && orderItem.getOrderID() > 0) {
			Map parameters = new HashMap();
			parameters.put("orderID", Integer.toString(orderItem.getOrderID()));
			OrderProcessor parentProcessor =  bof
					.createObject(OrderProcessor.class);
			order = parentProcessor.loadOrderFromID(parameters);
			request.setAttribute("order", order);
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("orderItemAttributeEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Orders",
				"Order Details", "Order Items", "Order Item Details",
				"Order Item Attributes", "Order Item Attribute Details" });
		request.setAttribute("breadcrumbs", bcs);

		// Create childSubNavigation collection
		Collection childsn = Arrays.asList(new String[] { "Order Item Details",
				"Order Item Attributes", "Add Order Item Attribute",
				"Order Item Attribute Details" });
		request.setAttribute("childSubNavigation", childsn);

		request.setAttribute("orderItemAttribute", orderItemAttribute);
		request.setAttribute("orderItemAttributeAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
