package com.softslate.commerce.administrator.order;

import java.util.Arrays;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.order.State;
import com.softslate.commerce.businessobjects.order.StateProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/StateEditForm.do". Loads in the current values of the record
 * being edited and populates a <code>StateAddEditForm</code> object.
 * 
 * @author David Tobey
 * 
 */
public class StateEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(StateAddFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		StateAddEditForm baseForm = (StateAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		StateProcessor processor = (StateProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"stateProcessorImplementer");
		State state = processor.loadStateFromID(PropertyUtils.describe(form));
		BeanUtils.copyProperties(baseForm, state);

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("stateEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "States",
				"State Details" });
		request.setAttribute("breadcrumbs", bcs);

		request.setAttribute("state", state);
		request.setAttribute("stateAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
