package com.softslate.commerce.administrator.ordermanagement;

import java.util.Collection;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Extension of the Struts <code>Action</code> class with common methods used
 * by order management actions in the Administrator.
 * 
 * @author David Tobey
 * 
 */
public class BaseOrderManagementAction extends Action {

	static Log log = LogFactory.getLog(BaseOrderManagementAction.class);

	public String wrapOrderManagementAction(ActionForm form) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		BaseForm baseForm = (BaseForm) form;

		OrderManagementUtils orderManagement = (OrderManagementUtils) baseForm
				.getBusinessObjectFactory().createUtilsObject(
						"orderManagementUtilsImplementer");

		synchronized (AppUtils.getSessionLock(baseForm.getRequest().getSession())) {

			// Save the session's order, customer and coupon codes in case this
			// admin user is also in the shopper side.
			Order savedOrder = baseForm.getUser().getOrder();
			Customer savedCustomer = baseForm.getUser().getCustomer();
			Collection savedCouponCodes = baseForm.getUser()
					.getEnteredCouponCodes();

			try {
				return orderManagement.wrapAction(this, baseForm);
			} finally {

				// Set the order back to what it was in case this admin
				// user was also on the shopper side.
				baseForm.getUser().setOrder(savedOrder);
				baseForm.getUser().setCustomer(savedCustomer);
				baseForm.getUser().setEnteredCouponCodes(savedCouponCodes);
			}
		}
	}

	/**
	 * Override this with a subclass to implement order management
	 * functionality.
	 * 
	 * @param baseForm
	 * @return A Map. If populated with an element whose key is "returnForward"
	 *         the calling method will find the forward and return it without
	 *         processing shipping or tax.
	 * @throws Exception
	 * @see OrderManagementUtils#wrapAction(BaseOrderManagementAction,
	 *      com.softslate.commerce.customer.core.BaseForm)
	 */
	public Map processOrderManagementAction(BaseForm baseForm) throws Exception {
		return null;
	}
}
