package com.softslate.commerce.administrator.ordermanagement;

import java.util.Collection;
import java.util.Map;

import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * Interface for utility methods used by Struts Action classes implement Ajax
 * functionality.
 * <p>
 * When the application needs to create an instance that implements
 * <code>AjaxUtils</code>, it calls
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory#createUtilsObject} ,
 * which finds the name of the Java class to instantiate from the
 * "ajaxUtilsImplemeter" setting in the <code>appComponents.properties</code>
 * file.
 * <p>
 * The default "ajaxUtilsImplemeter" is
 * {@link com.softslate.commerce.administrator.ordermanagement.OrderManagementUtilsImpl}.
 * 
 * @author David Tobey
 * 
 */
public interface OrderManagementUtils {

	/**
	 * Given a Collection of valid shipping options and an Order object,
	 * attempts to find the shippping option within the collection that was
	 * previously selected for the order.
	 * 
	 * @param options
	 * @param order
	 * @param baseForm
	 * @return the shipping option, or null if it could not be found
	 */
	abstract Map findPreviousShippingOption(Collection options, Order order,
			BaseForm baseForm);

	/**
	 * Given an order object, return a Collection of all the coupon codes used
	 * by OrderDiscount objects under the Order.
	 * 
	 * @param order
	 * @return a Collection of strings
	 */
	abstract Collection findCouponCodes(Order order);

	/**
	 * Given an order and another order to compare it to, return a Map
	 * describing the differences between the two orders.
	 * 
	 * @param order
	 * @param originalOrder
	 * @param newOrderItems
	 * @return A Map representing the differences between the two orders.
	 */
	abstract Map findOrderChanges(Order order, Order originalOrder,
			Collection newOrderItems);

	/**
	 * Populate {@code baseForm} with user, order, and customer
	 * objects all derived from its {@code orderID} property,
	 * and pass {@code baseForm} to {@code action}'s
	 * {@link BaseOrderManagementAction#processOrderManagementAction(com.softslate.commerce.customer.core.BaseForm)}
	 * method.
	 * 
	 * @param action object whose processOrderManagementAction method to call
	 * @param baseForm request context
	 * @return Struts forward name, <i>e.g.,</i> "success" or "exception"
	 * @see BaseOrderManagementAction
	 */
	abstract String wrapAction(BaseOrderManagementAction action,
			BaseForm baseForm) throws Exception;
}
