package com.softslate.commerce.administrator.product;

import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.CategoryProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.product.ProductInitializer;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/CategoryEditDelete.do". Processes a request to delete and
 * edit multiple records in the system. These requests are generated when the
 * adminstrator is deleting or editing multiple records simultaneously while in
 * edit mode.
 * 
 * @author David Tobey
 */
public class CategoryEditDeleteAction extends BaseAction {

    static Log log = LogFactory.getLog(CategoryEditDeleteAction.class);

    public ActionForward execute(ActionMapping mapping, ActionForm form,
            HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        if (log.isDebugEnabled())
            log.debug("Starting action.");
        CategoryForm baseForm = (CategoryForm) form;

        // Prepare parameters for the edit/delete processing, and process
        BusinessObjectFactory bof = (BusinessObjectFactory) request
                .getAttribute("businessObjectFactory");
        CategoryProcessor processor =  bof
                .createObject(CategoryProcessor.class);
        Map parameters = PropertyUtils.describe(form);
        for (int j = 0; j < baseForm.getDisplayFields().length; j++) {
            String displayField = baseForm.getDisplayFields()[j];
            
            // Bug where the parentCategory field gets renamed to parent
            if (displayField.equals("parent")) {
            	displayField = "parentCategory";
            	baseForm.getDisplayFields()[j] = "parentCategory";
            }
            parameters.put(displayField, PropertyUtils.getProperty(baseForm,
                    displayField));
        }
        parameters.put(baseForm.getIdField(), PropertyUtils.getProperty(
                baseForm, baseForm.getIdField()));
        parameters.put("deleteID", baseForm.getDeleteID());
        processor.editDeleteCategories(parameters);

        // Load all categories for the parent category drop down.
        Collection categories = processor.loadAllCategories();
        request.setAttribute("categories", categories);

        // Reload category tree and builtin categories in the application scope.
        ProductInitializer pi = new ProductInitializer();
        pi.setServlet(servlet);
        pi.initializeCategoryTree(bof);
        pi.initializeBuiltInCategories(bof);

        // Set the success message
        ActionMessages messages = new ActionMessages();
        messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
                "messages.recordsSaved"));
        saveMessages(request, messages);

        return mapping.findForward("success");
    }
}
