package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.Discount;
import com.softslate.commerce.businessobjects.product.DiscountProcessor;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.businessobjects.product.SKU;
import com.softslate.commerce.businessobjects.product.SKUProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * "/administrator/DiscountEditForm.do". Loads in the current values of the
 * record being edited and populates a <code>DiscountAddEditForm</code>
 * object.
 * 
 * @author David Tobey
 */
public class DiscountEditFormAction extends BaseAction {

	static Log log = LogFactory.getLog(DiscountEditFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		DiscountAddEditForm baseForm = (DiscountAddEditForm) form;
		baseForm.initializeProperties(mapping, request);

		// Get the record to edit
		DiscountProcessor processor = (DiscountProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"discountProcessorImplementer");
		Discount discount = null;
		if (baseForm.getDiscountID() != null) {
			discount = processor.loadDiscountFromID(PropertyUtils.describe(form));
		}
		if (discount == null && baseForm.getCode() != null) {
			discount = processor.loadDiscountFromCode(PropertyUtils.describe(form));
		}
		BeanUtils.copyProperties(baseForm, discount);
		baseForm.splitValues();

		// Load the parent product if an id is present
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		Product product = null;
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {
			ProductProcessor productProcessor =  bof
					.createObject(ProductProcessor.class);
			Map parameters = new HashMap();
			parameters.put("productID", baseForm.getProductID());
			product = productProcessor.loadProductFromID(parameters);
			request.setAttribute("product", product);
		}

		// Load the parent sku if an id is present
		SKU sku = null;
		if (baseForm.getSkuID() != null && baseForm.getSkuID().length() > 0) {
			SKUProcessor skuProcessor =  bof
					.createObject(SKUProcessor.class);
			Map parameters = new HashMap();
			parameters.put("skuID", baseForm.getSkuID());
			sku = skuProcessor.loadSKUFromID(parameters);
			request.setAttribute("sku", sku);
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("discountEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Navigation
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products",
					"Product Details", "Discounts", "Discount Details" });
			request.setAttribute("breadcrumbs", bcs);

			// Create child childSubNavigation collection
			Collection childsn = Arrays
					.asList(new String[] { "Discount Details",
							"Discount Ranges", "Add Discount Range" });
			request.setAttribute("childSubNavigation", childsn);
			
			request.setAttribute("childSubNavigation", childsn);
		} else if (baseForm.getSkuID() != null
				&& baseForm.getSkuID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "SKUs",
					"SKU Details", "Discounts", "Discount Details" });
			request.setAttribute("breadcrumbs", bcs);

			// Create child childSubNavigation collection
			Collection childsn = Arrays
					.asList(new String[] { "Discount Details",
							"Discount Ranges", "Add Discount Range" });
			request.setAttribute("childSubNavigation", childsn);
		} else {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Discounts",
					"Discount Details" });
			request.setAttribute("breadcrumbs", bcs);
		}
		request.setAttribute("discount", discount);
		request.setAttribute("discountAddEditForm", baseForm);
		return mapping.findForward("success");
	}
}
