package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.DiscountRangeProcessor;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.businessobjects.product.SKU;
import com.softslate.commerce.businessobjects.product.SKUProcessor;
import com.softslate.commerce.businessobjects.product.Discount;
import com.softslate.commerce.businessobjects.product.DiscountProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to the main administrator view panel for
 * options. Used for requests to "/administrator/DiscountRange.do".
 * <p>
 * The <code>execute</code> method of this class populates a
 * <code>Collection</code> named <code>itemCollection</code> in the request
 * scope containing the items to be displayed. Next, it sets a request scope
 * variable named <code>editPermission</code> that determines if the current
 * user has permission to edit the items. Finally, it creates a
 * <code>Collection</code> named <code>breadcrumbs</code> containing the
 * links and labels of each breadcrumb for display.
 * 
 * @author David Tobey
 * 
 */
public class DiscountRangeAction extends BaseAction {

	static Log log = LogFactory.getLog(DiscountRangeAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		DiscountRangeForm baseForm = (DiscountRangeForm) form;
		baseForm.setProperties(request);
		request.setAttribute("mainForm", baseForm);

		// Handle the quick-add form
		DiscountRangeAddEditForm addForm = (DiscountRangeAddEditForm) request
				.getAttribute("discountRangeAddEditForm");
		if (addForm == null
				|| (request.getAttribute("reset") != null && request
						.getAttribute("reset").equals("true"))) {
			addForm = new DiscountRangeAddEditForm();
		}
		addForm.initializeProperties(mapping, request);
		request.setAttribute("discountRangeAddEditForm", addForm);

		// Get the collection of items to display
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		DiscountRangeProcessor processor =  bof
				.createObject(DiscountRangeProcessor.class);
		Map itemsAndCount = processor.loadDiscountRangesAndCount(PropertyUtils
				.describe(baseForm));
		request.setAttribute("options", itemsAndCount.get("items"));
		request.setAttribute("itemCollection", itemsAndCount.get("items"));
		request.setAttribute("itemCount", itemsAndCount.get("itemCount"));
		request.setAttribute("assignedItems", itemsAndCount
				.get("assignedItems"));

		// Load the parent product if an id is present
		Product product = null;
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {
			ProductProcessor productProcessor =  bof
					.createObject(ProductProcessor.class);
			Map parameters = new HashMap();
			parameters.put("productID", baseForm.getProductID());
			product = productProcessor.loadProductFromID(parameters);
			request.setAttribute("product", product);
		}

		// Load the parent sku if an id is present
		SKU sku = null;
		if (baseForm.getSkuID() != null && baseForm.getSkuID().length() > 0) {
			SKUProcessor skuProcessor =  bof
					.createObject(SKUProcessor.class);
			Map parameters = new HashMap();
			parameters.put("skuID", baseForm.getSkuID());
			sku = skuProcessor.loadSKUFromID(parameters);
			request.setAttribute("sku", sku);
		}

		// Load the parent discount if an id is present
		Discount discount = null;
		if (baseForm.getDiscountID() != null
				&& baseForm.getDiscountID().length() > 0) {
			DiscountProcessor discountProcessor =  bof
					.createObject(DiscountProcessor.class);
			Map parameters = new HashMap();
			parameters.put("discountID", baseForm.getDiscountID());
			discount = discountProcessor.loadDiscountFromID(parameters);
			request.setAttribute("discount", discount);
		}

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("discountEdit")) {
			request.setAttribute("editPermission", "false");
			baseForm.setEditMode("false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Navigation
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products",
					"Product Details", "Discounts", "Discount Details",
					"Discount Ranges" });
			request.setAttribute("breadcrumbs", bcs);

			// Create child childSubNavigation collection
			Collection childsn = Arrays
					.asList(new String[] { "Discount Details",
							"Discount Ranges", "Add Discount Range" });
			request.setAttribute("childSubNavigation", childsn);
		} else if (baseForm.getSkuID() != null
				&& baseForm.getSkuID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "SKUs",
					"SKU Details", "Discounts", "Discount Details",
					"Discount Ranges" });
			request.setAttribute("breadcrumbs", bcs);

			// Create child childSubNavigation collection
			Collection childsn = Arrays
					.asList(new String[] { "Discount Details",
							"Discount Ranges", "Add Discount Range" });
			request.setAttribute("childSubNavigation", childsn);
		} else {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Discounts",
					"Discount Details", "Discount Ranges" });
			request.setAttribute("breadcrumbs", bcs);
		}

		// Forward according to the outputMode of the request
		if (baseForm.getOutputMode().equals("print")) {
			request.setAttribute("outputMode", "print");
			baseForm.setOutputMode("screen");
			return mapping.findForward("successSimpleLayout");
		} else if (baseForm.getOutputMode().equals("excel")) {
			request.setAttribute("outputMode", "excel");
			baseForm.setOutputMode("screen");
			response.setHeader("Content-Type", "application/vnd.ms-excel");
			response.setHeader("Content-Disposition",
					"attachment; filename=options.xls");
			return mapping.findForward("successSimpleLayout");
		}
		return mapping.findForward("success");
	}
}
