package com.softslate.commerce.administrator.product;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.product.LuceneIndexer;
import com.softslate.commerce.businessobjects.product.Manufacturer;
import com.softslate.commerce.businessobjects.product.ManufacturerProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.product.ProductInitializer;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/ManufacturerEdit.do". Processes a request to edit a record
 * within the system.
 * 
 * @author David Tobey
 */
public class ManufacturerEditAction extends BaseAction {

	static Log log = LogFactory.getLog(ManufacturerEditAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		ManufacturerAddEditForm baseForm = (ManufacturerAddEditForm) form;

		// Edit the record
		ManufacturerProcessor processor = (ManufacturerProcessor) baseForm
				.getBusinessObjectFactory().createObject(
						"manufacturerProcessorImplementer");
		Map results = processor.editManufacturer(baseForm.mapOfIncomingParams());

		// Successful update
		if (results == null || results.get("resultCode") == null || results.get("resultCode").equals("0")) {

			// Reload manufacturers in the application scope.
			ProductInitializer pi = new ProductInitializer();
			pi.setServlet(servlet);
			pi.initializeManufacturers(baseForm.getBusinessObjectFactory());

			// entity.jsp wants the updated record in the request scope
			Manufacturer manufacturer = (Manufacturer) results.get("updatedRecord");
			request.setAttribute("manufacturer", manufacturer);

			// Set the success message
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordSaved"));
			saveMessages(baseForm.getRequest(), baseForm.getMessages());

			// Optionally update the Lucene index
			updateAdvancedSearchIndex(baseForm, manufacturer.getManufacturerID());
		} else if (results != null && results.get("existingRecord") != null) {
			Manufacturer existingRecord = (Manufacturer) results.get("existingRecord");
			String existingCode = StringEscapeUtils.escapeXml11(existingRecord.getCode());
			request.setAttribute("okNotToEscapeMessages", Boolean.TRUE);
			baseForm.getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.resultCode" + results.get("resultCode"), results.get("duplicateField"),
							existingCode));
			baseForm.getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.editExistingRecord", "ManufacturerEditForm.do?manufacturerID="
							+ existingRecord.getManufacturerID(), results.get("duplicateField"), existingCode));
		}

		// Save errors and messages
		saveErrors(baseForm.getRequest(), baseForm.getErrors());
		saveMessages(baseForm.getRequest(), baseForm.getMessages());
		return mapping.findForward("success");
	}
	
	protected void updateAdvancedSearchIndex(ManufacturerAddEditForm baseForm, int manufacturerID) {
		if (baseForm.getSettings().getValue("advancedSearchOn").equals("1") && baseForm.getUpdateAdvancedSearchIndex()) {
			Map indexerResults = null;
			try {
				LuceneIndexer indexer = baseForm.getBusinessObjectFactory().createObject(LuceneIndexer.class);
				String index = (String) baseForm.getSettings().getValue("luceneIndexDir");
				String indexDir = getServlet().getServletContext().getRealPath(index);
				indexerResults = indexer.addEditManufacturer(manufacturerID, indexDir);
				if (indexerResults.get("message") != null && ((String) indexerResults.get("message")).length() > 0) {
					baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.detail", indexerResults.get("message")));
				}
			} catch (Exception e) {
				log.error("Exception reindexing manufacturer: ", e);
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.detail", e.toString()));
			}
		}
	}
}
