package com.softslate.commerce.businessobjects.core;

import java.util.Collection;
import java.util.Map;
import java.util.Iterator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.order.OrderProcessor;

/**
 * An example of a class which may be run from the command line to execute
 * processing within the system. See the accompanying
 * <code>commandLineExample.sh</code> file for a Unix shell script that may be
 * used to fire off the example.
 * 
 * This example uses the <code>CommandLineSupport</code> class to initialize
 * all of the system objects such as the <code>BusinessObjectFactory</code>
 * and the <code>DAOFactory</code>. It then executes a query for the number
 * of recent orders (same query that is displayed on the welcome screen of the
 * Administrator) and outputs the results, as an example of how to access the
 * system.
 * 
 * @author dtobey
 * 
 */
public class CommandLineExample extends CommandLineSupport {

	static Log log = LogFactory.getLog(CommandLineExample.class);

	/**
	 * Main method, which gets invoked when executing this class from the
	 * command line.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {

		// Log the start of the process
		if (log.isInfoEnabled())
			log.info("Starting command line process.");

		if (args == null || args.length != 0) {
			if (log.isErrorEnabled())
				log.error("Wrong number of arguments. 0 arguments needed.");
			System.out
					.println("Usage: ./commandLineExample.sh");
		}

		// Initilize the system objects
		Map systemObjects = initializeSystem("Command Line Admin User");
		if (systemObjects == null) {
			if (log.isErrorEnabled())
				log.error("Initialization failed ... exiting.");
			return;
		}

		// Unpack the system objects needed by this process
		BusinessObjectFactory bof = (BusinessObjectFactory) systemObjects
				.get("bof");
		// Properties appSettings = (Properties)
		// systemObjects.get("appSettings");
		// Properties appComponents = (Properties) systemObjects
		// .get("appComponents");
		// DAOFactory daoFactory = (DAOFactory) systemObjects.get("daoFactory");
		// Settings settings = (Settings) systemObjects.get("settings");
		// User user = (User) systemObjects.get("user");

		// Perform the processing
		if (log.isInfoEnabled())
			log.info("Doing command line processing.");

		try {

			OrderProcessor orderProcessor =  bof
					.createObject(OrderProcessor.class);
			Collection statistics = orderProcessor.loadDashboardStatistics();

			if (log.isInfoEnabled())
				log.info("Outputting processing results.");

			Iterator i = statistics.iterator();
			while (i.hasNext()) {
				if (log.isInfoEnabled())
					log.info(i.next());
			}

		} catch (Exception e) {
			if (log.isErrorEnabled())
				log.error("Command line processing failed ... exiting.", e);
			return;
		}

		// Log the end of process
		if (log.isInfoEnabled())
			log.info("Ending command line process.");
	}
}
