package com.softslate.commerce.businessobjects.events;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.ApplicationMessage;
import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;

public abstract class BaseTrigger extends BaseBusinessProcessor implements Runnable {

	static Log log = LogFactory.getLog(BaseTrigger.class);

	abstract public void processTrigger() throws Exception;

	private boolean isActive = true;
	private boolean swallowExceptions = false;
	private boolean executeInSeparateThread = false;

	public boolean getIsActive() {
		return isActive;
	}

	public void setIsActive(boolean isActive) {
		this.isActive = isActive;
	}

	public boolean getSwallowExceptions() {
		return swallowExceptions;
	}

	public void setSwallowExceptions(boolean swallowExceptions) {
		this.swallowExceptions = swallowExceptions;
	}

	public boolean getExecuteInSeparateThread() {
		return executeInSeparateThread;
	}

	public void setExecuteInSeparateThread(boolean executeInSeparateThread) {
		this.executeInSeparateThread = executeInSeparateThread;
	}

	public void process() throws Exception {
		String className = getClass().getName();
		if (getIsActive()) {
			if (getExecuteInSeparateThread()) {
				log.info("Processing " + className + " in new Thread.");
				processInSeparateThread();
			} else {
				try {
					log.info("Processing " + className + " in current Thread.");
					processInSameThread();
				} catch (Exception e) {
					if (!getSwallowExceptions()) {
						log.error("Exception processing " + className + ": ", e);
						throw e;
					} else {
						log.error("Exception processing " + className + " (swallowing): ", e);
					}
				}
			}
		} else {
			log.info(className + " not processed as it is not active.");
		}
	}

	public void processInSeparateThread() {
		new Thread(this).start();
	}

	public void processInSameThread() throws Exception {
		processTrigger();
	}

	public void run() {
		try {
			processTrigger();
		} catch (Exception e) {
			log.error("Exception processing " + getClass().getName() + " in separate thread: ", e);
		}
	}

	public void addMessage(Map results, ApplicationMessage.Level level, String message) {
		if (results == null) {
			results = new HashMap();
		}
		if (results.get("applicationMessages") == null) {
			results.put("applicationMessages", new ArrayList());
		}
		Collection<ApplicationMessage> triggerMessages = (Collection<ApplicationMessage>) results
				.get("applicationMessages");
		triggerMessages.add(new ApplicationMessage(level, message));
	}

}
