package com.softslate.commerce.businessobjects.events;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.email.Email;
import com.softslate.commerce.businessobjects.email.EmailSender;
import com.softslate.commerce.businessobjects.email.Template;
import com.softslate.commerce.businessobjects.email.TemplateParser;
import com.softslate.commerce.businessobjects.email.TemplateProcessor;
import com.softslate.commerce.businessobjects.order.Order;

public abstract class EmailNotificationTrigger extends BaseTrigger {
	protected String templateCode;
	protected String emailType;
	protected Template template;
	protected TemplateProcessor templateProcessor;
	protected TemplateParser templateParser;
	protected Email email;
	protected Map<String, Object> extraTemplateObjects;
	protected BusinessObjectFactory bof;

	abstract void initializeTemplateCodeAndType() throws Exception;

	public void processTrigger() throws Exception {
		initializeTemplateCodeAndType();
		if (templateCode != null && emailType != null) {
			template = templateProcessor.loadTemplateFromCode(templateCode);
			email = createEmail();
			extraTemplateObjects = createExtraTemplateObjects();
			templateParser = createTemplateParser();
			if (templateParser.parseEmail()) {
				sendEmail();
			} else {
				if (log.isErrorEnabled())
					log.error("Error parsing template: " + template.getCode());
			}
		}
	}

	protected Email createEmail() throws Exception {
		Email email =  bof.createObject(Email.class);
		email.setType(emailType);
		email.setTemplate(template);
		return email;
	}

	protected Email createEmailFromOrder(Order order) throws Exception {
		Email email =  bof.createObject(Email.class);
		email.setCustomer(order.getCustomer());
		email.setOrder(order);
		email.setToEmails(order.getEmail1());
		email.setType(emailType);
		email.setTemplate(template);
		return email;
	}

	protected Map<String, Object> createExtraTemplateObjects() throws Exception {
		return null;
	}

	protected TemplateParser createTemplateParser() throws Exception {
		TemplateParser templateParser =  bof.createObject(TemplateParser.class);
		templateParser.initialize(template, email, extraTemplateObjects);
		return templateParser;
	}

	protected void sendEmail() throws Exception {
		EmailSender sender =  bof.createObject(EmailSender.class);
		sender.setEmail(email);
		sender.send();
	}

}