package com.softslate.commerce.businessobjects.order;

import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.order.OrderShippingRuleDAO;
import com.softslate.commerce.daos.order.OrderShippingRuleGatewayDAO;

/**
 * Processes business logic having to do with order shipping rules. Created in the Struts layer in response to requests
 * for viewing, editing, adding, and deleting order shipping rules.
 * <p>
 * <code>BasicOrderShippingRuleProcessor</code> is the default implementation of the
 * <code>OrderShippingRuleProcessor</code> interface for the application.
 * 
 * @author David Tobey
 * 
 */
public class BasicOrderShippingRuleProcessor extends BaseBusinessProcessor implements OrderShippingRuleProcessor {

	static Log log = LogFactory.getLog(BasicOrderShippingRuleProcessor.class);

	public Map loadOrderShippingRulesAndCount(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading order shipping rules and count");
		OrderShippingRuleGatewayDAO gatewayDAO = getDaoFactory().createDAO(OrderShippingRuleGatewayDAO.class);
		return gatewayDAO.loadOrderShippingRulesAndCount(parameters);
	}

	public Map addOrderShippingRule(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName() + ": Adding order shipping rule: " + code);
		}
		OrderShippingRule orderShippingRule = getBusinessObjectFactory().createObject(OrderShippingRule.class);
		BeanUtils.copyProperties(orderShippingRule, parameters);
		Order order = (Order) getBusinessObjectFactory().createObject("orderImplementer");
		order.setOrderID(orderShippingRule.getOrderID());
		orderShippingRule.setOrder(order);
		if (orderShippingRule.getOrderItemID() != null && orderShippingRule.getOrderItemID().intValue() != 0) {
			OrderItem orderItem = getBusinessObjectFactory().createObject(OrderItem.class);
			orderItem.setOrderItemID(orderShippingRule.getOrderItemID().intValue());
			orderShippingRule.setOrderItem(orderItem);
		}
		OrderShippingRuleDAO dao = (OrderShippingRuleDAO) getDaoFactory().createDAO("orderShippingRuleDAOImplementer");
		dao.setOrderShippingRule(orderShippingRule);
		dao.insertOrderShippingRule(true);
		return null;
	}

	public OrderShippingRule loadOrderShippingRuleFromID(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading order shipping rule from id");
		OrderShippingRule orderShippingRule = getBusinessObjectFactory().createObject(OrderShippingRule.class);
		OrderShippingRuleDAO dao = (OrderShippingRuleDAO) getDaoFactory().createDAO("orderShippingRuleDAOImplementer");
		BeanUtils.copyProperties(orderShippingRule, parameters);
		dao.setOrderShippingRule(orderShippingRule);
		dao.loadOrderShippingRuleFromID();
		return orderShippingRule;
	}

	public Map editOrderShippingRule(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName() + ": Editing order shipping rule: " + code);
		}
		OrderShippingRule orderShippingRule = getBusinessObjectFactory().createObject(OrderShippingRule.class);
		BeanUtils.copyProperties(orderShippingRule, parameters);
		Order order = (Order) getBusinessObjectFactory().createObject("orderImplementer");
		order.setOrderID(orderShippingRule.getOrderID());
		orderShippingRule.setOrder(order);
		if (orderShippingRule.getOrderItemID() != null && orderShippingRule.getOrderItemID().intValue() != 0) {
			OrderItem orderItem = getBusinessObjectFactory().createObject(OrderItem.class);
			orderItem.setOrderItemID(orderShippingRule.getOrderItemID().intValue());
			orderShippingRule.setOrderItem(orderItem);
		}
		OrderShippingRuleDAO dao = (OrderShippingRuleDAO) getDaoFactory().createDAO("orderShippingRuleDAOImplementer");
		dao.setOrderShippingRule(orderShippingRule);
		dao.updateOrderShippingRule(true);
		return null;
	}

	public Map editDeleteOrderShippingRules(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple order shipping rules");
		OrderShippingRuleGatewayDAO dao = (OrderShippingRuleGatewayDAO) getDaoFactory()
				.createDAO("orderShippingRuleGatewayDAOImplementer");
		Map results = dao.editDeleteOrderShippingRules(parameters, getBusinessObjectFactory());
		return results;
	}

	public void deleteOrderShippingRule(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName() + ": Delete order shipping rule: " + code);
		}
		OrderShippingRule orderShippingRule = getBusinessObjectFactory().createObject(OrderShippingRule.class);
		BeanUtils.copyProperties(orderShippingRule, parameters);
		OrderShippingRuleDAO dao = (OrderShippingRuleDAO) getDaoFactory().createDAO("orderShippingRuleDAOImplementer");
		dao.setOrderShippingRule(orderShippingRule);
		dao.deleteOrderShippingRule(true);
		return;
	}
}
