package com.softslate.commerce.businessobjects.order;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with order shipping rules. Created in the Struts layer in response
 * to requests for viewing, editing, adding, and deleting order shipping rules.
 * <p>
 * When the application needs to create an instance that implements <code>OrderShippingRuleProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}<!-- --> finds the name of the Java class to
 * instantiate from the "orderShippingRuleProcessorImplementer" setting in the <code>appComponents.properties</code>
 * file.
 * <p>
 * The default "orderShippingRuleProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.order.OrderShippingRuleProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface OrderShippingRuleProcessor extends BusinessProcessor {

	abstract Map loadOrderShippingRulesAndCount(Map parameters)
            throws Exception;

	abstract Map addOrderShippingRule(Map parameters) throws Exception;

	abstract OrderShippingRule loadOrderShippingRuleFromID(Map parameters)
            throws Exception;

	abstract Map editOrderShippingRule(Map parameters) throws Exception;

	abstract Map editDeleteOrderShippingRules(Map parameters)
            throws Exception;

	abstract void deleteOrderShippingRule(Map parameters) throws Exception;
}
