package com.softslate.commerce.businessobjects.product;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.product.ManufacturerDAO;
import com.softslate.commerce.daos.product.ManufacturerGatewayDAO;
import com.softslate.commerce.daos.product.ProductGatewayDAO;

/**
 * Processes business logic having to do with Manufacturers. Created in the
 * Struts layer in response to requests for viewing, editing, adding, and
 * deleting Manufacturers.
 * <p>
 * <code>BasicManufacturerProcessor</code> is the default implementation of the
 * <code>ManufacturerProcessor</code> interface for the application.
 * 
 * @author David Tobey
 * 
 */
public class BasicManufacturerProcessor extends BaseBusinessProcessor implements ManufacturerProcessor {

	static Log log = LogFactory.getLog(BasicManufacturerProcessor.class);

	public Collection loadAllManufacturers() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading all Manufacturers");
		ManufacturerGatewayDAO gatewayDAO = (ManufacturerGatewayDAO) getDaoFactory().createDAO(
				"manufacturerGatewayDAOImplementer");
		return gatewayDAO.loadAllManufacturers();
	}

	public Collection loadActiveManufacturers() throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading active Manufacturers");
		ManufacturerGatewayDAO gatewayDAO = (ManufacturerGatewayDAO) getDaoFactory().createDAO(
				"manufacturerGatewayDAOImplementer");
		return gatewayDAO.loadActiveManufacturers();
	}

	public Map loadManufacturersAndCount(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading Manufacturers and count");
		ManufacturerGatewayDAO gatewayDAO = (ManufacturerGatewayDAO) getDaoFactory().createDAO(
				"manufacturerGatewayDAOImplementer");
		return gatewayDAO.loadManufacturersAndCount(parameters);
	}

	public Map addManufacturer(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName() + ": Adding manufacturer: " + code);
		}
		Map results = new HashMap();
		Manufacturer manufacturer =  getBusinessObjectFactory().createObject(Manufacturer.class);
		BeanUtils.copyProperties(manufacturer, parameters);
		ManufacturerDAO dao =  getDaoFactory().createDAO(ManufacturerDAO.class);
		dao.setManufacturer(manufacturer);
		dao.insertManufacturer(true);
		results.put("newRecord", dao.getManufacturer());
		return results;
	}

	public Manufacturer loadManufacturerFromID(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading manufacturer from id");
		Manufacturer manufacturer =  getBusinessObjectFactory().createObject(Manufacturer.class);
		ManufacturerDAO dao =  getDaoFactory().createDAO(ManufacturerDAO.class);
		BeanUtils.copyProperties(manufacturer, parameters);
		dao.setManufacturer(manufacturer);
		dao.loadManufacturerFromID();
		return dao.getManufacturer();
	}

	public Manufacturer loadManufacturerFromCode(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading manufacturer from code");
		Manufacturer manufacturer =  getBusinessObjectFactory().createObject(Manufacturer.class);
		ManufacturerDAO dao =  getDaoFactory().createDAO(ManufacturerDAO.class);
		BeanUtils.copyProperties(manufacturer, parameters);
		dao.setManufacturer(manufacturer);
		dao.loadManufacturer();
		return dao.getManufacturer();
	}

	public Map editManufacturer(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("code");
			log.info(getUser().getAdministrator().getUserName() + ": Editing manufacturer: " + code);
		}
		Map results = new HashMap();
		Manufacturer manufacturer = null;
		if (parameters.get("manufacturerID") != null) {
			manufacturer = loadManufacturerFromID(parameters);
		}
		if (manufacturer == null && parameters.get("code") != null) {
			manufacturer = loadManufacturerFromCode(parameters);
		}
		BeanUtils.copyProperties(manufacturer, parameters);
		ManufacturerDAO dao =  getDaoFactory().createDAO(ManufacturerDAO.class);
		dao.setManufacturer(manufacturer);
		dao.updateManufacturer(true);
		results.put("updatedRecord", dao.getManufacturer());
		return results;
	}

	public Map editDeleteManufacturers(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple Manufacturers");
		ManufacturerGatewayDAO dao = (ManufacturerGatewayDAO) getDaoFactory().createDAO(
				"manufacturerGatewayDAOImplementer");
		Map results = dao.editDeleteManufacturers(parameters, getBusinessObjectFactory());
		return results;
	}

	public void deleteManufacturer(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Deleting manufacturer");
		Manufacturer manufacturer =  getBusinessObjectFactory().createObject(Manufacturer.class);
		BeanUtils.copyProperties(manufacturer, parameters);
		ManufacturerDAO dao =  getDaoFactory().createDAO(ManufacturerDAO.class);
		dao.setManufacturer(manufacturer);
		dao.deleteManufacturer(true);
		return;
	}

	public Manufacturer getManufacturerFromCode(Manufacturer manufacturer) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading manufacturer: " + manufacturer.getCode());
		return getManufacturerFromCode(manufacturer, true);
	}

	public Manufacturer getManufacturerFromCode(Manufacturer manufacturer, boolean loadProducts) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading manufacturer: " + manufacturer.getCode());
		ManufacturerDAO manufacturerDAO =  getDaoFactory().createDAO(ManufacturerDAO.class);
		manufacturerDAO.setManufacturer(manufacturer);
		// Load the manufacturer itself, preserving the original paging params.
		manufacturerDAO.loadManufacturer();
		if (manufacturerDAO.getManufacturer() == null) {
			return manufacturer;
		}
		// Preserve the pagination parameters
		manufacturerDAO.getManufacturer().setItemsPerPage(manufacturer.getItemsPerPage());
		manufacturerDAO.getManufacturer().setFirstRow(manufacturer.getFirstRow());
		manufacturer = manufacturerDAO.getManufacturer();
		// Load the manufacturer's products.
		if (loadProducts) {
			if (log.isDebugEnabled())
				log.debug("Loading manufacturer's product list: " + manufacturer.getCode());
			ProductList productList =  getBusinessObjectFactory().createObject(ProductList.class);
			productList.setProductListType(ProductListBean.MANUFACTURER);
			productList.setManufacturerID(manufacturer.getManufacturerID());
			productList.setItemsPerPage(manufacturer.getItemsPerPage());
			productList.setFirstRow(manufacturer.getFirstRow());
			productList.setItemCount(manufacturer.getItemCount());
			ProductGatewayDAO productGatewayDAO = (ProductGatewayDAO) getDaoFactory().createDAO(
					"productGatewayDAOImplementer");
			productGatewayDAO.loadProductList(productList);
			productList.addFormatting(getUser().getLocale(), (String) getSettings().getValue("currencyCode"));
			manufacturer.setProductListCollection(productList.getProductListCollection());
			manufacturer.setItemCount(productList.getItemCount());
		}
		return manufacturer;
	}

}
