package com.softslate.commerce.businessobjects.product;

import java.util.Collection;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with categories. Created in
 * the Struts layer in response to requests for viewing, editing, adding, and
 * deleting categories.
 * <p>
 * When the application needs to create an instance that implements
 * <code>CategoryProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "categoryProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "categoryProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.CategoryProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface CategoryProcessor extends BusinessProcessor {
	abstract Collection loadAllCategories() throws Exception;

	abstract Map loadCategoriesAndCount(Map parameters) throws Exception;

	abstract Map addCategory(Map parameters) throws Exception;

	abstract Category loadCategoryFromID(Map parameters) throws Exception;

	abstract Category loadCategoryFromCode(Map parameters) throws Exception;

	abstract Map editCategory(Map parameters) throws Exception;

	abstract Map editDeleteCategories(Map parameters) throws Exception;

	abstract Map deleteCategory(Map parameters) throws Exception;

	abstract Collection loadSettings(Map properties) throws Exception;

	abstract Collection processSettings(Map properties) throws Exception;

	abstract Map sortCategoriesByName(Map parameters) throws Exception;
	
	abstract Map sortCategoriesByCode(Map parameters) throws Exception;

	abstract int loadMaxCategoryID() throws Exception;

	abstract Collection loadCategoriesForIndexing(int firstCategoryID,
			int lastCategoryID) throws Exception;
}
