package com.softslate.commerce.businessobjects.product;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;
import com.softslate.commerce.businessobjects.order.OrderItem;

/**
 * Object representing a discount within the system. <code>DiscountBean</code>
 * is the default implementation of the <code>Discount</code> interface for
 * the application.
 * <p>
 * The database stores discounts in the <code>sscDiscount</code> table, and
 * this interface can be used to represent a single row of that table. The
 * Hibernate configuration file <code>DiscountBean.hbm.xml</code> maps that
 * table's columns to the properties in this object.
 * 
 * @author David Tobey
 */
@Audited
@AuditTable("AudDiscount")
public class DiscountBean extends BaseBusinessObject implements Discount,
		Serializable {

	private static final long serialVersionUID = -9201592616051778757L;

	private int discountID = 0;

	private String name = null;

	private String code = null;

	private String description = null;

	private boolean isActive = true;

	private String usersAppliesType = null;

	private String usersAppliesValue = null;

	private String itemsAppliesType = null;

	private String itemsAppliesValue = null;

	private String cartItemsType = null;

	private String cartItemsValue = null;

	private String rewardType = null;

	private Double reward = null;

	private String couponCode = null;

	private String startDate = null;

	private String expirationDate = null;

	private Integer timesUsed = null;

	private Integer maxTimesUsed = null;

	private boolean isTimesPerCustomer = true;

	private Integer orderItemID = null;

	private OrderItem orderItem = null;

	private int discountOrder = 0;

	private String extra1 = null;

	private String extra2 = null;

	private String extra3 = null;

	private Collection skuDiscounts = null;

	private Collection discountRanges = null;

	private Collection orderItems;

	private Integer totalQuantity = new Integer(0);

	private Double totalAmount = new Double(0);

	private Double totalWeight = new Double(0);

	private Map discountInformation = new HashMap();

	public String getCouponCode() {
		return couponCode;
	}

	public void setCouponCode(String couponCode) {
		this.couponCode = couponCode;
	}

	public boolean getIsTimesPerCustomer() {
		return isTimesPerCustomer;
	}

	public void setIsTimesPerCustomer(boolean isTimesPerCustomer) {
		this.isTimesPerCustomer = isTimesPerCustomer;
	}

	public String getItemsAppliesType() {
		return itemsAppliesType;
	}

	public void setItemsAppliesType(String itemsAppliesType) {
		this.itemsAppliesType = itemsAppliesType;
	}

	public String getItemsAppliesValue() {
		return itemsAppliesValue;
	}

	public void setItemsAppliesValue(String itemsAppliesValue) {
		this.itemsAppliesValue = itemsAppliesValue;
	}

	public String getUsersAppliesType() {
		return usersAppliesType;
	}

	public void setUsersAppliesType(String usersAppliesType) {
		this.usersAppliesType = usersAppliesType;
	}

	public String getUsersAppliesValue() {
		return usersAppliesValue;
	}

	public void setUsersAppliesValue(String usersAppliesValue) {
		this.usersAppliesValue = usersAppliesValue;
	}

	public String getCartItemsType() {
		return cartItemsType;
	}

	public void setCartItemsType(String cartItemsType) {
		this.cartItemsType = cartItemsType;
	}

	public String getCartItemsValue() {
		return cartItemsValue;
	}

	public void setCartItemsValue(String cartItemsValue) {
		this.cartItemsValue = cartItemsValue;
	}

	public Double getReward() {
		return reward;
	}

	public void setReward(Double reward) {
		this.reward = reward;
	}

	public String getRewardType() {
		return rewardType;
	}

	public void setRewardType(String rewardType) {
		this.rewardType = rewardType;
	}

	public void setActive(boolean isActive) {
		this.isActive = isActive;
	}

	public String getCode() {
		return code;
	}

	public void setCode(String code) {
		this.code = code;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public int getDiscountID() {
		return discountID;
	}

	public void setDiscountID(int discountID) {
		this.discountID = discountID;
	}

	public int getDiscountOrder() {
		return discountOrder;
	}

	public void setDiscountOrder(int discountOrder) {
		this.discountOrder = discountOrder;
	}

	public String getExpirationDate() {
		return expirationDate;
	}

	public void setExpirationDate(String expirationDate) {
		this.expirationDate = expirationDate;
	}

	public String getExtra1() {
		return extra1;
	}

	public void setExtra1(String extra1) {
		this.extra1 = extra1;
	}

	public String getExtra2() {
		return extra2;
	}

	public void setExtra2(String extra2) {
		this.extra2 = extra2;
	}

	public String getExtra3() {
		return extra3;
	}

	public void setExtra3(String extra3) {
		this.extra3 = extra3;
	}

	public boolean getIsActive() {
		return isActive;
	}

	public void setIsActive(boolean isActive) {
		this.isActive = isActive;
	}

	public Integer getMaxTimesUsed() {
		return maxTimesUsed;
	}

	public void setMaxTimesUsed(Integer maxTimesUsed) {
		this.maxTimesUsed = maxTimesUsed;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getStartDate() {
		return startDate;
	}

	public void setStartDate(String startDate) {
		this.startDate = startDate;
	}

	public Integer getTimesUsed() {
		return timesUsed;
	}

	public void setTimesUsed(Integer timesUsed) {
		this.timesUsed = timesUsed;
	}

	public Integer getOrderItemID() {
		return orderItemID;
	}

	public void setOrderItemID(Integer orderItemID) {
		this.orderItemID = orderItemID;
	}

	public OrderItem getOrderItem() {
		return orderItem;
	}

	public void setOrderItem(OrderItem orderItem) {
		this.orderItem = orderItem;
	}

	public Collection getSkuDiscounts() {
		return skuDiscounts;
	}

	public void setSkuDiscounts(Collection skuDiscounts) {
		this.skuDiscounts = skuDiscounts;
	}

	public Collection getDiscountRanges() {
		return discountRanges;
	}

	public void setDiscountRanges(Collection discountRanges) {
		this.discountRanges = discountRanges;
	}

	public Collection getOrderItems() {
		return orderItems;
	}

	public void setOrderItems(Collection orderItems) {
		this.orderItems = orderItems;
	}

	public Integer getTotalQuantity() {
		return totalQuantity;
	}

	public void setTotalQuantity(Integer totalQuantity) {
		this.totalQuantity = totalQuantity;
	}

	public Double getTotalAmount() {
		return totalAmount;
	}

	public void setTotalAmount(Double totalAmount) {
		this.totalAmount = totalAmount;
	}

	public Double getTotalWeight() {
		return totalWeight;
	}

	public void setTotalWeight(Double totalWeight) {
		this.totalWeight = totalWeight;
	}

	public Map getDiscountInformation() {
		return discountInformation;
	}

	public void setDiscountInformation(Map discountInformation) {
		this.discountInformation = discountInformation;
	}

}
