package com.softslate.commerce.businessobjects.product;

import java.util.List;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with skus. Created in the
 * Struts layer in response to requests for viewing, editing, adding, and
 * deleting skus.
 * <p>
 * When the application needs to create an instance that implements
 * <code>SKUProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "skuProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "skuProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.SKUProcessor}.
 * 
 * @author David Tobey
 */
public interface SKUProcessor extends BusinessProcessor {

	abstract Map loadSKUsAndCount(Map parameters) throws Exception;

	abstract Map addSKU(Map parameters) throws Exception;

	abstract SKU loadSKUFromID(Map parameters) throws Exception;

	abstract Map editSKU(Map parameters) throws Exception;

	abstract Map editDeleteSKUs(Map parameters) throws Exception;

	abstract void deleteSKU(Map parameters) throws Exception;

	abstract Map buildProductSKUs(int productID, List<Integer> includedAttributeIds, List<Integer> includedOptionIds)
			throws Exception;

	abstract Map buildAllProductSKUs() throws Exception;

	abstract Map buildAllAttributeSKUs() throws Exception;

	abstract int loadMaxSKUID() throws Exception;

	abstract Map postChangingSKUEvent(SKU sku, Map parameters) throws Exception;

	abstract SKU loadSKUFromID(int skuID) throws Exception;

	abstract SKU loadSKUFromNumber(Map parameters) throws Exception;

	abstract SKU loadSKUFromNumber(String skuNumber) throws Exception;

	abstract Map editSKUAssignment(Map parameters) throws Exception;

}
