package com.softslate.commerce.customer.core;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;

import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.product.SearchList;

/**
 * <code>FormUtilsImpl</code> is the default implementation of the
 * <code>FormUtils</code> interface for the application.
 * 
 * @author David Tobey
 * @see com.softslate.commerce.customer.core.FormUtils
 * 
 */
public class FormUtilsImpl implements FormUtils {

	static Log log = LogFactory.getLog(FormUtilsImpl.class);

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.FormUtilsInt#stringExists(com.softslate
	 * .commerce.customer.core.BaseDynaForm, java.lang.String)
	 */
	public boolean stringExists(BaseDynaForm form, String property) {
		try {
			String value = (String) form.get(property);
			if (value == null || value.length() == 0)
				return false;
		} catch (IllegalArgumentException iae) {
			return false;
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.FormUtilsInt#twoPropertiesEqual(
	 * com.softslate.commerce.customer.core.BaseDynaForm, java.lang.String,
	 * java.lang.String)
	 */
	public boolean twoPropertiesEqual(BaseDynaForm form, String property1, String property2) {
		try {
			String value1 = (String) form.get(property1);
			String value2 = (String) form.get(property2);
			if (value1 == null || value2 == null) {
				return false;
			}
			if (!value1.equals(value2))
				return false;
		} catch (IllegalArgumentException iae) {
			return false;
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.FormUtilsInt#stringValueGreaterThan
	 * (com.softslate.commerce.customer.core.BaseDynaForm, java.lang.String,
	 * int)
	 */
	public boolean stringValueGreaterThan(BaseDynaForm form, String property, int value) {
		try {
			String parameter = (String) form.get(property);
			if (parameter == null) {
				return false;
			}
			int thisValue = Integer.parseInt(parameter);
			if (log.isDebugEnabled())
				log.debug("Checking if: " + thisValue + " is greater than: " + value);
			if (thisValue > value)
				return true;
		} catch (IllegalArgumentException iae) {
			log.error(iae.toString(), iae);
			return false;
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.FormUtilsInt#stringValueLessThan
	 * (com.softslate.commerce.customer.core.BaseDynaForm, java.lang.String,
	 * int)
	 */
	public boolean stringValueLessThan(BaseDynaForm form, String property, int value) {
		try {
			String parameter = (String) form.get(property);
			if (parameter == null) {
				return false;
			}
			int thisValue = Integer.parseInt(parameter);
			if (log.isDebugEnabled())
				log.debug("Checking if: " + thisValue + " is less than: " + value);
			if (thisValue < value)
				return true;
		} catch (IllegalArgumentException iae) {
			log.error(iae.toString(), iae);
			return false;
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.FormUtilsInt#stringArrayHasAnElement
	 * (com.softslate.commerce.customer.core.BaseDynaForm, java.lang.String)
	 */
	public boolean stringArrayHasAnElement(BaseDynaForm form, String property) {
		try {
			String value = (String) form.get(property, 0);
			if (value == null || value.length() == 0)
				return false;
		} catch (ArrayIndexOutOfBoundsException aiobe) {
			return false;
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.softslate.commerce.customer.core.FormUtilsInt#
	 * stringArrayValuesGreaterThan
	 * (com.softslate.commerce.customer.core.BaseDynaForm, java.lang.String,
	 * int)
	 */
	public boolean stringArrayValuesGreaterThan(BaseDynaForm form, String property, int value) {
		try {
			String[] paramValues = (String[]) form.get(property);
			if (paramValues == null) {
				return false;
			}
			for (int i = 0; i < paramValues.length; i++) {
				try {
					int thisValue = Integer.parseInt(paramValues[i]);
					if (log.isDebugEnabled())
						log.debug("Checking if: " + thisValue + " is greater than: " + value);
					if (thisValue > value) {
						return true;
					}
				} catch (NumberFormatException nfe) {
					log.error(nfe.toString(), nfe);
				}
			}
		} catch (ArrayIndexOutOfBoundsException aiobe) {
			return false;
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.FormUtilsInt#stringArrayHasAnElement
	 * (java.lang.String[])
	 */
	public boolean stringArrayHasAnElement(String[] paramValues) {
		if (paramValues == null) {
			return false;
		}
		try {
			String value = paramValues[0];
			if (value.length() == 0)
				return false;
		} catch (ArrayIndexOutOfBoundsException aiobe) {
			return false;
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.softslate.commerce.customer.core.FormUtilsInt#
	 * stringArrayValuesGreaterThan(java.lang.String[], int)
	 */
	public boolean stringArrayValuesGreaterThan(String[] paramValues, int value) {
		if (paramValues == null) {
			return false;
		}
		try {
			for (int i = 0; i < paramValues.length; i++) {
				try {
					int thisValue = Integer.parseInt(paramValues[i]);
					if (log.isDebugEnabled())
						log.debug("Checking if: " + thisValue + " is greater than: " + value);
					if (thisValue > value) {
						return true;
					}
				} catch (NumberFormatException nfe) {
					if (log.isDebugEnabled())
						log.debug(nfe.getMessage());
				}
			}
		} catch (ArrayIndexOutOfBoundsException aiobe) {
			return false;
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.FormUtilsInt#capitalize(java.lang
	 * .String)
	 */
	public String capitalize(String stringToCapitalize) {
		if (stringToCapitalize == null) {
			return null;
		}
		StringBuffer sb = new StringBuffer(stringToCapitalize);
		char firstChar = sb.charAt(0);
		char cappedFirstChar = Character.toUpperCase(firstChar);
		sb.setCharAt(0, cappedFirstChar);
		return sb.toString();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.FormUtilsInt#isValidEmail(java.lang
	 * .String)
	 */
	public boolean isValidEmail(String email) {
		if (email == null) {
			return false;
		}
		try {
			InternetAddress.parse(email);
		} catch (AddressException ae) {
			return false;
		}
		return email.matches(".+@.+\\..+");
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.FormUtilsInt#isValidPhone(java.lang
	 * .String)
	 */
	public boolean isValidPhone(String phone) {
		if (phone == null) {
			return false;
		}
		phone = phone.replaceAll("\\D", "");
		return (phone.length() > 9);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.FormUtilsInt#isValidPostalCode(java
	 * .lang.String, java.lang.String)
	 */
	public boolean isValidPostalCode(String postalCode, String country) {
		if (country != null && country.equals("US")) {
			return postalCode.matches("(\\d\\d\\d\\d\\d)|(\\d\\d\\d\\d\\d-\\d\\d\\d\\d)");
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.FormUtilsInt#isInteger(java.lang
	 * .String)
	 */
	public boolean isInteger(String value) {
		if (value == null) {
			return false;
		}
		try {
			Integer.parseInt(value);
			return true;
		} catch (NumberFormatException nfe) {
			log.info("Not an integer: " + value);
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.FormUtilsInt#isCreditCardValid(java
	 * .lang.String)
	 */
	public boolean isCreditCardValid(String creditCardNumber) {
		int sum = 0;
		int numberOfDigits = creditCardNumber.length();
		int parity = numberOfDigits % 2;
		for (int i = 0; i < numberOfDigits; i++) {
			int digit = Integer.parseInt(creditCardNumber.substring(i, i + 1));
			if (i % 2 == parity) {
				digit = digit * 2;
			}
			if (digit > 9) {
				digit = digit - 9;
			}
			sum = sum + digit;
		}
		return (sum % 10) == 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.FormUtilsInt#doCardTypeAndNumberMatch
	 * (java.lang.String, java.lang.String)
	 */
	public boolean doCardTypeAndNumberMatch(String type, String number) {
		if (type == null || number == null) {
			return false;
		}
		if (type.equals("VISA")
				&& ((number.length() != 13 && number.length() != 16) || !number.substring(0, 1).equals("4"))) {
			return false;
		} else if (type.equals("DISCOVER") && (number.length() != 16 || !number.substring(0, 4).equals("6011"))) {
			return false;
		} else if (type.equals("AMEX")
				&& (number.length() != 15 || (!number.substring(0, 2).equals("34") && !number.substring(0, 2).equals(
						"37")))) {
			return false;
		} else if (type.equals("MASTERCARD") && (number.length() != 16 || !number.substring(0, 1).equals("5"))) {
			return false;
		}
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.softslate.commerce.customer.core.FormUtilsInt#deriveCardType(java
	 * .lang.String)
	 */
	public String deriveCardType(String number) {
		if (number == null || number.length() < 4) {
			return null;
		}
		if ((number.length() == 13 || number.length() == 16) && number.substring(0, 1).equals("4")) {
			return "VISA";
		} else if (number.length() == 16 && number.substring(0, 4).equals("6011")) {
			return "DISCOVER";
		} else if (number.length() == 15
				&& (number.substring(0, 2).equals("34") || number.substring(0, 2).equals("37"))) {
			return "AMEX";
		} else if (number.length() == 16 && number.substring(0, 1).equals("5")) {
			return "MASTERCARD";
		} else if (number.length() == 16 && number.substring(0, 2).equals("35")) {
			return "JCB";
		} else if (number.length() == 14
				&& (number.substring(0, 2).equals("36") || number.substring(0, 2).equals("38")
						|| number.substring(0, 3).equals("300") || number.substring(0, 3).equals("305"))) {
			return "DINER'S CLUB";
		}
		return null;
	}

	public Character containsCharacter(String string, String characters) {
		if (string == null || characters == null) {
			return null;
		}
		char[] chars = characters.toCharArray();
		for (int i = 0; i < chars.length; i++) {
			if (string.indexOf(chars[i]) != -1) {
				return new Character(chars[i]);
			}
		}
		return null;
	}

	public String stringArrayAsURLParam(String parameterName, String[] stringArray) throws Exception {
		StringBuffer sb = new StringBuffer("");
		if (stringArray != null) {
			for (int i = 0; i < stringArray.length; i++) {
				sb.append("&" + parameterName + "=" + java.net.URLEncoder.encode(stringArray[i], "UTF-8"));
			}
		}
		return sb.toString();
	}

	public void eliminateDuplicateFacets(SearchList searchList) {

		// Required to work around an apparent Struts bug where the facet array
		// has two of each facet.
		if (searchList != null && searchList.getFacets() != null) {
			Collection nonDupedFacets = new HashSet();
			String[] facets = searchList.getFacets();
			for (int i = 0; i < facets.length; i++) {
				nonDupedFacets.add(facets[i]);
			}
			searchList.setFacets((String[]) nonDupedFacets.toArray(new String[0]));
		}
		return;
	}

	public void removeNullsFromMap(Map map) {
		if (map == null) {
			return;
		}
		Collection keysToRemove = new ArrayList();
		Iterator i = map.keySet().iterator();
		while (i.hasNext()) {
			String key = (String) i.next();
			if (map.get(key) == null) {
				keysToRemove.add(key);
			}
		}
		Iterator j = keysToRemove.iterator();
		while (j.hasNext()) {
			map.remove(j.next());
		}
	}

	public void removeAllNotInRequestFromMap(Map map, HttpServletRequest request) {
		if (map == null) {
			return;
		}
		Collection keysToRemove = new ArrayList();
		Iterator i = map.keySet().iterator();
		while (i.hasNext()) {
			String key = (String) i.next();
			if (request.getParameter(key) == null) {
				keysToRemove.add(key);
			}
		}
		Iterator j = keysToRemove.iterator();
		while (j.hasNext()) {
			map.remove(j.next());
		}
	}

	public void removePrefixedKeysFromMap(Map map, String prefix) {
		if (map == null || prefix == null) {
			return;
		}
		Collection keysToRemove = new ArrayList();
		Iterator i = map.keySet().iterator();
		while (i.hasNext()) {
			String key = (String) i.next();
			if (key.startsWith(prefix) || map.get(key) == null) {
				keysToRemove.add(key);
			}
		}
		Iterator j = keysToRemove.iterator();
		while (j.hasNext()) {
			map.remove(j.next());
		}
	}

	public String html5OrXhtml(HttpServletRequest request, Settings settings) {
		String html5OrXhtml = "default-html5";
		String baseLayout = settings.getValue("baseLayout");
		String commonLayout = settings.getValue("commonLayout");
		String defaultLayout = settings.getValue("defaultLayout");
		String currentLayout = (String) request.getAttribute("currentLayout");
		if (("default-xhtml").equals(currentLayout)) {
			html5OrXhtml = "default-xhtml";
		} else if (("default-html5").equals(currentLayout)) {
			html5OrXhtml = "default-html5";
		} else if (("default-xhtml").equals(defaultLayout)) {
			html5OrXhtml = "default-xhtml";
		} else if (("default-html5").equals(defaultLayout)) {
			html5OrXhtml = "default-html5";
		} else if (("default-xhtml").equals(commonLayout)) {
			html5OrXhtml = "default-xhtml";
		} else if (("default-html5").equals(commonLayout)) {
			html5OrXhtml = "default-html5";
		} else if (("default-xhtml").equals(baseLayout)) {
			html5OrXhtml = "default-xhtml";
		} else if (("default-html5").equals(baseLayout)) {
			html5OrXhtml = "default-html5";
		}
		return html5OrXhtml;
	}
}
