package com.softslate.commerce.customer.core;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * A servlet filter that locks the user's browser session so no other actions
 * with a session lock can take place until the first request finishes.
 * 
 * @author David Tobey
 */
public class SessionLockFilter implements Filter {

	private static Log log = LogFactory.getLog(SessionLockFilter.class);

	public void init(FilterConfig filterConfig) throws ServletException {
		return;
	}

	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws IOException, ServletException {

		// Must cast incoming request to HttpServletRequest
		HttpServletRequest req = null;
		try {
			req = (HttpServletRequest) request;
		} catch (ClassCastException cce) {
			if (log.isInfoEnabled())
				log.info("Incoming request not of type HttpServletRequest?: "
						+ cce);
		}

		// Apply the session lock to anything that modifies the cart.
		if (req.getServletPath().equals("/CartItemEdit.do")
				|| req.getServletPath().equals("/CartAdd.do")
				|| req.getServletPath().equals("/CartClear.do")
				|| req.getServletPath().equals("/CartCoupon.do")
				|| req.getServletPath().equals("/CartAddMultiple.do")
				|| req.getServletPath().equals("/OrderFormCoupon.do")
				|| req.getServletPath().equals("/OrderProcess.do")
				|| req.getServletPath().equals("/Register.do")
				|| req.getServletPath().equals("/SaveCart.do")
				|| req.getServletPath().startsWith("/Update")
				|| req.getServletPath().endsWith("Pickup.do")
				|| req.getServletPath().startsWith("/Checkout")) {
			log.info("Applying all-encompassing session lock for request: "
					+ req.getServletPath());
			synchronized (AppUtils.getSessionLock(req.getSession())) {
				chain.doFilter(request, response);
				return;
			}
		}

		chain.doFilter(request, response);
	}

	public void destroy() {
	}

}
