package com.softslate.commerce.customer.customer;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.CustomerProcessor;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseForm;
import com.softslate.commerce.customer.order.CartAddAction;

/**
 * Struts action class used for requests to remove a saved cart by a customer.
 * Used for requests to "/SavedCartRemove.do".
 * <p>
 * The <code>execute</code> method of this class invokes
 * {@link com.softslate.commerce.businessobjects.customer.CustomerProcessor#removeSavedCart(Map)}
 * to change the status of the order from "Saved" to "Removed".
 * 
 * @author David Tobey
 * 
 */
public class SavedCartRemoveAction extends CartAddAction {
	static Log log = LogFactory.getLog(SavedCartRemoveAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		BaseForm baseForm = (BaseForm) form;
		synchronized (AppUtils.getSessionLock(baseForm.getRequest().getSession())) {

			// Preserve the original request so the customer is redirected
			// back to the main saved carts page after logging in
			request.setAttribute("originalRequest", "/AccountSavedCarts.do");

			// If the user is not logged in, it's an error.
			if (baseForm.getUser().getCustomer() == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.userNotLoggedIn"));
				saveErrors(baseForm.getRequest(),
						(ActionMessages) baseForm.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("notLoggedIn");
			}

			// Remove the saved cart. This will change the order's status to
			// "Removed"
			CustomerProcessor customerProcessor = (CustomerProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"customerProcessorImplementer");
			customerProcessor.removeSavedCart(PropertyUtils.describe(baseForm));

			return mapping.findForward("success");
		}
	}
}
