package com.softslate.commerce.customer.customer;

import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.Settings;
import com.softslate.commerce.businessobjects.customer.CustomerWishList;
import com.softslate.commerce.businessobjects.customer.CustomerWishListProcessor;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.businessobjects.product.ProductSetting;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class used for requests to add a product to a wish list. Used
 * for requests to "/WishListAddProduct.do".
 * 
 * @author Jason McWilliams
 * 
 */
public class WishListAddProductAction extends BaseAction {
	static Log log = LogFactory.getLog(WishListAddProductAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		WishListForm baseForm = (WishListForm) form;
		synchronized (AppUtils.getSessionLock(request.getSession())) {

			// Preserve the original request so the customer is redirected
			// back here after logging in
			request
					.setAttribute("originalRequest", request.getRequestURI()
							+ "?productCode=" + baseForm.getProductCode()
							+ "&customerWishListID="
							+ baseForm.getCustomerWishListID());

			// If the user is not logged in, it's an error.
			if (baseForm.getUser().getCustomer() == null) {
				baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("messages.wishListMustBeLoggedIn"));
				this.saveMessages(request, baseForm.getMessages());
				if (log.isInfoEnabled() && !baseForm.getMessages().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getMessages().toString());
				return mapping.findForward("notLoggedIn");
			}

			// If the system is not allowing wish lists, error
			Settings settings = (Settings) servlet.getServletContext()
					.getAttribute("settings");
			String allowCustomerWishLists = (String) settings
					.getValue("allowCustomerWishLists");
			if (allowCustomerWishLists == null
					|| !allowCustomerWishLists.equals("1")) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.wishListsNotAllowed"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}

			// Load the product
			if (log.isInfoEnabled())
				log.info("Loading the product.");
			ProductProcessor productProcessor = (ProductProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"productProcessorImplementer");
			Product product =  baseForm.getBusinessObjectFactory()
					.createObject(Product.class);
			product.setCode(baseForm.getProductCode());
			product = productProcessor.getProductFromCode(product);

			// If that fails, it's an error
			if (product.getProductID() == 0 || !product.getIsActive()) {
				baseForm.getErrors().add(
						ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.productNotFound", product
								.getCode()));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}

			// Check that this product is allowed to be in a wish list
			if (log.isInfoEnabled())
				log.info("Checking the product settings.");
			Map productSettings = product.getProductSettings();
			if (productSettings != null && !productSettings.isEmpty()
					&& productSettings.containsKey("allowCustomerWishLists")
					&& productSettings.get("allowCustomerWishLists") != null) {
				ProductSetting productSetting = (ProductSetting) productSettings
						.get("allowCustomerWishLists");
				if (productSetting.getValue().equals("0")) {
					baseForm.getErrors().add(
							ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage(
									"errors.wishListsProductNotAllowed"));
					saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
							.getErrors());
					if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
						log.info("Validation error(s): "
								+ baseForm.getErrors().toString());
					return mapping.findForward("failure");
				}
			}

			// Find the wish list, or, if the customer doesn't have one, create
			// it
			if (log.isInfoEnabled())
				log.info("Finding the wish list.");
			CustomerWishListProcessor customerWishListProcessor = (CustomerWishListProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"customerWishListProcessorImplementer");
			Map properties = PropertyUtils.describe(baseForm);
			String wishListDefaultName = baseForm.getMessage("customer",
					"wishList.defaultName", null);
			String wishListDefaultDescription = baseForm.getMessage("customer",
					"wishList.defaultDescription", null);
			properties.put("name", wishListDefaultName);
			properties.put("description", wishListDefaultDescription);
			CustomerWishList customerWishList = customerWishListProcessor
					.customerGetWishList(properties, true);
			baseForm.getRequest().setAttribute("customerWishList", customerWishList);

			// Error if wish list wasn't found
			if (customerWishList == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.wishListNotFound"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}

			// Make sure we have a collection
			if (log.isInfoEnabled())
				log.info("Adding the product to the wish list.");
			if (customerWishList.getCustomerWishListProducts() == null) {
				Set customerWishListProducts = new LinkedHashSet();
				customerWishList
						.setCustomerWishListProducts(customerWishListProducts);
			}

			// Make sure the user hasn't added too many items to their wish
			// list
			int maxProductsPerWishLists = 0;
			try {
				String maxProductsPerWishListsString = (String) settings
						.getValue("maxProductsPerWishLists");
				maxProductsPerWishLists = Integer
						.parseInt(maxProductsPerWishListsString);
			} catch (Exception e) {
				log.error(e.toString(), e);
			}
			if (maxProductsPerWishLists <= customerWishList
					.getCustomerWishListProducts().size()) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.wishListsMaxProducts"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}

			// Add it
			customerWishListProcessor.customerAddProductToWishList(
					customerWishList, product);

			// Add a message to tell the user it worked
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("messages.wishListProductAdded"));
			saveMessages(baseForm.getRequest(), baseForm.getMessages());
			return mapping.findForward("success");
		}
	}
}
