package com.softslate.commerce.customer.payment;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.Customer;
import com.softslate.commerce.businessobjects.payment.AuthorizeNetAPIProcessor;
import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseAction;

import net.authorize.api.contract.v1.CustomerPaymentProfileMaskedType;

/**
 * 
 * @author Jason McWilliams
 * 
 */
public class AuthorizeNetDeleteProfileAction extends BaseAction {
	static Log log = LogFactory.getLog(AuthorizeNetDeleteProfileAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {

		if (log.isInfoEnabled())
			log.info("Starting action.");
		AuthorizeNetDeleteProfileForm baseForm = (AuthorizeNetDeleteProfileForm) form;
		synchronized (AppUtils.getSessionLock(request.getSession())) {

			// Check the Authorize.net settings to make sure the action is allowed.
			String authorizeNetAllowManageProfiles = baseForm.getSettings().getValue("authorizeNetAllowManageProfiles");
			if (request.getAttribute("authorizeNetProfilesOnAndEnabled") == null
					|| !(Boolean) request.getAttribute("authorizeNetProfilesOnAndEnabled")
					|| authorizeNetAllowManageProfiles == null || !authorizeNetAllowManageProfiles.equals("1")) {
				log.info("CIM is not an active payment processor.");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesNotAllowed"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}
		

			// If the user is not logged in, it's an error.
			if (baseForm.getUser().getCustomer() == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.userNotLoggedIn"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				log.info("Validation error(s): " + baseForm.getErrors().toString());
				return mapping.findForward("notLoggedIn");
			}

			// Check Settings
			String authorizeNetIntegrationMethod = (String) baseForm.getSettings()
					.getValue("authorizeNetIntegrationMethod");
			if ("AcceptHostedIframe".equals(authorizeNetIntegrationMethod)) {
				log.info("authorizeNetIntegrationMethod can't be 'AcceptHostedIframe'");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesNotAllowedAPI"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			// Separate setting to allow deletes - store owners might not wish to allow it.
			String authorizeNetAllowDeleteProfiles = (String) baseForm.getSettings().getValue(
					"authorizeNetAllowDeleteProfiles");
			if (!"1".equals(authorizeNetAllowDeleteProfiles)) {
				if (log.isInfoEnabled())
					log.info("Delete not allowed by settings.");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesUnableToDeleteProfile"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			// Retrieve the payment profile from Authorize.net
			Customer customer = baseForm.getUser().getCustomer();
			AuthorizeNetAPIProcessor apiProcessor = (AuthorizeNetAPIProcessor) baseForm.getBusinessObjectFactory()
					.createObject("authorizeNetAPIProcessorImplementer");
			Map result = null;
			CustomerPaymentProfileMaskedType paymentProfile = apiProcessor
					.retrievePaymentProfile(customer.getCimProfileID(), baseForm.getCimPaymentProfileID());
			if (paymentProfile == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesGetProfileError"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			// Delete it
			try {
				Map parameters = PropertyUtils.describe(baseForm);
				parameters.put("customerProfileID", customer.getCimProfileID());
				parameters.put("paymentProfileID", baseForm.getCimPaymentProfileID());
				result = apiProcessor.deletePaymentProfile(parameters);
			} catch (Exception ex) {
				log.error("Exception deleting payment profile: ", ex);
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesUnableToDeleteProfile"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			}

			if (result == null || result.get("resultCode") == null
					|| ("Error").equalsIgnoreCase((String) result.get("resultCode"))) {
				log.error("Error deleting payment profile" + result.get("resultCode") + ": "
						+ result.get("resultMessage"));
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.cimManageProfilesUnableToDeleteProfile"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm.getErrors());
				return mapping.findForward("failure");
			} else {
				// Add a message to tell the user it worked
				baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("messages.cimProfileDeleted"));
				saveMessages(baseForm.getRequest(), baseForm.getMessages());
				baseForm.getRequest().setAttribute("addressForm", baseForm);
				return mapping.findForward("success");
			}
		}
	}
}
