package com.softslate.commerce.daos.product;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.product.DiscountRange;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscDiscountRange</code> table of the
 * database.
 * 
 * @author David Tobey
 */
public class DiscountRangeDAOHibernate extends BaseDAO implements DAOInterface,
        DiscountRangeDAO {

    static Log log = LogFactory.getLog(DiscountRangeDAOHibernate.class);

    private DiscountRange discountRange = null;

    public DiscountRange getDiscountRange() {
        return (this.discountRange);
    }

    public void setDiscountRange(DiscountRange discountRange) {
        this.discountRange = discountRange;
    }

    public void loadDiscountRangeFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            DiscountRange discountRange = (DiscountRange) session.get(
                    DiscountRange.class, new Integer(getDiscountRange()
                            .getDiscountRangeID()));
            PropertyUtils.copyProperties(getDiscountRange(), discountRange);
            setDiscountRange(discountRange);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertDiscountRange() throws DataAccessException {
        insertDiscountRange(false);
    }

    public void insertDiscountRange(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.save(getDiscountRange());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
            evictCache();
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateDiscountRange() throws DataAccessException {
        updateDiscountRange(false);
    }

    public void updateDiscountRange(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getDiscountRange());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteDiscountRange() throws DataAccessException {
        deleteDiscountRange(false);
    }

    public void deleteDiscountRange(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            setDiscountRange((DiscountRange) session.load(DiscountRange.class,
                    new Integer(getDiscountRange().getDiscountRangeID())));
            session.delete(getDiscountRange());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
            evictCache();
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
}
