package com.softslate.commerce.installer;

import java.util.Properties;

import javax.servlet.RequestDispatcher;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.customer.core.AppUtils;
import com.softslate.commerce.customer.core.BaseRequestProcessor;

/**
 * InstallerRequestProcessor subclasses the BaseRequestProcessor to check for
 * whether or not the installer is locked.
 * 
 * @author David Tobey
 */
public class InstallerRequestProcessor extends BaseRequestProcessor {

    static Log log = LogFactory.getLog(InstallerRequestProcessor.class);

    public static final String lockedForwardPath = "/installer/InstallerLocked.do";

    public static final String errorPath = "/installer/Error.do";

    public static final String customLayoutDirectory = "custom";

    /**
     * General-purpose preprocessing hook. Loads settings into the request and
     * session scopes.
     * 
     * @param request
     *            The servlet request we are processing
     * @param response
     *            The servlet response we are creating
     */
    protected boolean processPreprocess(HttpServletRequest request,
            HttpServletResponse response) {
        if (log.isDebugEnabled()) {
            log
                    .debug("--------------------New Installer Request--------------------" + '\n');
        }

        // Set the custom layout directory to "custom" for custom templates
        synchronized (AppUtils.getSessionLock(request.getSession())) {
            request.getSession().setAttribute("currentLayout",
                    customLayoutDirectory);
        }

        // Let requests for the locked page and the error page go through.
        if (request.getServletPath().endsWith(errorPath)
                || request.getServletPath().endsWith(lockedForwardPath)) {
            return true;
        }

        // If this is a request for the installer program, make sure it isn't
        // locked.
		String appSettingsPropertiesFile = "appSettings.properties";
		if (System.getProperty("appSettings.configuration") != null) {
			appSettingsPropertiesFile = System
					.getProperty("appSettings.configuration");
		}
        Properties settings = AppUtils.loadProperties(
        		appSettingsPropertiesFile, servlet.getServletContext());
        if (log.isDebugEnabled())
            log.debug("lock property:" + settings.getProperty("lockInstaller"));
        if (settings != null
                && settings.getProperty("lockInstaller") != null
                && (settings.getProperty("lockInstaller").equals("locked")
                        || settings.getProperty("lockInstaller").equals("true") || settings
                        .getProperty("lockInstaller").equals("yes"))) {
            if (log.isDebugEnabled())
                log.debug("Installer is locked!" + '\n');
            RequestDispatcher rd = request
                    .getRequestDispatcher(lockedForwardPath);
            try {
                rd.forward(request, response);
                return (false);
            } catch (Exception se) {
                return (false);
            }
        }
        return (true);
    }
}
