CREATE TABLE sscOrder (
  orderID INT IDENTITY NOT NULL,
  customerID INT DEFAULT NULL,
  userName VARCHAR(100) DEFAULT NULL,
  orderNumber INT DEFAULT NULL,
  trackingNumber VARCHAR(100) DEFAULT NULL,
  total DECIMAL(10,4) DEFAULT NULL,
  shipping DECIMAL(10,4) DEFAULT NULL,
  tax DECIMAL(10,4) DEFAULT NULL,
  taxableSubtotal DECIMAL(10,4) DEFAULT NULL,
  subtotal DECIMAL(10,4) DEFAULT NULL,
  weight DECIMAL(10,4) DEFAULT NULL,
  quantity INT DEFAULT NULL,
  created VARCHAR(100) DEFAULT NULL,
  lastModified VARCHAR(100) DEFAULT NULL,
  completed VARCHAR(100) DEFAULT NULL,
  autoSavedCartToken VARCHAR(255) DEFAULT NULL,
  status VARCHAR(100) DEFAULT NULL,
  statusDetails VARCHAR(255) DEFAULT NULL,
  shippingMethod VARCHAR(255) DEFAULT NULL,
  firstName VARCHAR(100) DEFAULT NULL,
  lastName VARCHAR(100) DEFAULT NULL,
  organization VARCHAR(100) DEFAULT NULL,
  address1 VARCHAR(255) DEFAULT NULL,
  address2 VARCHAR(255) DEFAULT NULL,
  city VARCHAR(100) DEFAULT NULL,
  state VARCHAR(100) DEFAULT NULL,
  postalCode VARCHAR(100) DEFAULT NULL,
  country VARCHAR(100) DEFAULT NULL,
  phone1 VARCHAR(100) DEFAULT NULL,
  phone2 VARCHAR(100) DEFAULT NULL,
  email1 VARCHAR(100) DEFAULT NULL,
  email2 VARCHAR(100) DEFAULT NULL,
  paymentProfileID VARCHAR(100) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  extra4 VARCHAR(255) DEFAULT NULL,
  extra5 VARCHAR(255) DEFAULT NULL,
  notes TEXT DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderBean',
  CONSTRAINT PK_orderID PRIMARY KEY CLUSTERED 
  ( orderID ) ON [PRIMARY]
);

CREATE INDEX IDX_sscOrder_userName ON sscOrder (userName);
CREATE INDEX IDX_sscOrder_orderNumber ON sscOrder (orderNumber);
CREATE INDEX IDX_sscOrder_trackingNumber ON sscOrder (trackingNumber);
CREATE INDEX IDX_sscOrder_firstName ON sscOrder (firstName);
CREATE INDEX IDX_sscOrder_lastName ON sscOrder (lastName);
CREATE INDEX IDX_sscOrder_address1 ON sscOrder (address1);
CREATE INDEX IDX_sscOrder_postalCode ON sscOrder (postalCode);
CREATE INDEX IDX_sscOrder_phone1 ON sscOrder (phone1);
CREATE INDEX IDX_sscOrder_email1 ON sscOrder (email1);
CREATE INDEX IDX_sscO_autoSavedCartToken ON sscOrder (autoSavedCartToken);
CREATE INDEX IDX_sscOrder_completed ON sscOrder (completed);

CREATE TABLE sscOrderNumber (
  orderNumber INT IDENTITY NOT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderNumber',
  CONSTRAINT PK_orderNumber PRIMARY KEY CLUSTERED 
  ( orderNumber ) ON [PRIMARY]
);

CREATE TABLE sscOrderDelivery (
  orderDeliveryID INT IDENTITY NOT NULL,
  orderID INT NOT NULL DEFAULT '0',
  trackingNumber VARCHAR(100) DEFAULT NULL,
  total DECIMAL(10,4) DEFAULT NULL,
  shipping DECIMAL(10,4) DEFAULT NULL,
  tax DECIMAL(10,4) DEFAULT NULL,
  taxableSubtotal DECIMAL(10,4) DEFAULT NULL,
  subtotal DECIMAL(10,4) DEFAULT NULL,
  weight DECIMAL(10,4) DEFAULT NULL,
  quantity INT DEFAULT NULL,
  lastModified VARCHAR(100) DEFAULT NULL,
  delivered VARCHAR(100) DEFAULT NULL,
  status VARCHAR(100) DEFAULT NULL,
  statusDetails VARCHAR(255) DEFAULT NULL,
  firstName VARCHAR(100) DEFAULT NULL,
  lastName VARCHAR(100) DEFAULT NULL,
  organization VARCHAR(100) DEFAULT NULL,
  address1 VARCHAR(255) DEFAULT NULL,
  address2 VARCHAR(255) DEFAULT NULL,
  city VARCHAR(100) DEFAULT NULL,
  state VARCHAR(100) DEFAULT NULL,
  postalCode VARCHAR(100) DEFAULT NULL,
  country VARCHAR(100) DEFAULT NULL,
  phone1 VARCHAR(100) DEFAULT NULL,
  phone2 VARCHAR(100) DEFAULT NULL,
  email1 VARCHAR(100) DEFAULT NULL,
  email2 VARCHAR(100) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  extra4 VARCHAR(255) DEFAULT NULL,
  extra5 VARCHAR(255) DEFAULT NULL,
  notes TEXT DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderDeliveryBean',
  CONSTRAINT PK_orderDeliveryID PRIMARY KEY CLUSTERED 
  ( orderDeliveryID ) ON [PRIMARY],
  CONSTRAINT FK_order_orderDelivery FOREIGN KEY (
	orderID
  ) REFERENCES sscOrder (
	orderID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX IDX_sscOrderDelivery_trackingNumber ON sscOrderDelivery (trackingNumber);
CREATE INDEX IDX_sscOrderDelivery_firstName ON sscOrderDelivery (firstName);
CREATE INDEX IDX_sscOrderDelivery_lastName ON sscOrderDelivery (lastName);
CREATE INDEX IDX_sscOrderDelivery_address1 ON sscOrderDelivery (address1);
CREATE INDEX IDX_sscOrderDelivery_postalCode ON sscOrderDelivery (postalCode);
CREATE INDEX IDX_sscOrderDelivery_phone1 ON sscOrderDelivery (phone1);
CREATE INDEX IDX_sscOrderDelivery_email1 ON sscOrderDelivery (email1);

CREATE TABLE sscOrderItem (
  orderItemID INT IDENTITY NOT NULL,
  orderID INT NOT NULL DEFAULT '0',
  orderDeliveryID INT NOT NULL DEFAULT '0',
  total DECIMAL(10,4) DEFAULT NULL,
  weight DECIMAL(10,4) DEFAULT NULL,
  costTotal DECIMAL(10,4) DEFAULT NULL,
  quantity INT NOT NULL DEFAULT '1',
  productID INT NOT NULL DEFAULT '0',
  manufacturerID INT DEFAULT NULL,
  code VARCHAR(100) NOT NULL DEFAULT '',
  seoCode VARCHAR(255) DEFAULT NULL,
  name VARCHAR(255) DEFAULT NULL,
  isActive BIT NOT NULL DEFAULT '1',
  unitCost DECIMAL(10,4) DEFAULT NULL,
  unitPrice DECIMAL(10,4) DEFAULT NULL,
  altPrice DECIMAL(10,4) DEFAULT NULL,
  productWeight DECIMAL(10,4) DEFAULT NULL,
  isTaxed BIT NOT NULL DEFAULT '0',
  image VARCHAR(255) DEFAULT NULL,
  shortDescription VARCHAR(255) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  extra4 VARCHAR(255) DEFAULT NULL,
  extra5 VARCHAR(255) DEFAULT NULL,
  decrementedSKUs TEXT,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderItemBean',
  CONSTRAINT PK_orderItemID PRIMARY KEY CLUSTERED 
  ( orderItemID ) ON [PRIMARY],
  CONSTRAINT FK_order_orderItem FOREIGN KEY (
	orderID
  ) REFERENCES sscOrder (
	orderID
  ) ON DELETE NO ACTION,
  CONSTRAINT FK_orderDelivery_orderItem FOREIGN KEY (
	orderDeliveryID
  ) REFERENCES sscOrderDelivery (
	orderDeliveryID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX IDX_sscOrderItem_code ON sscOrderItem (code);
CREATE INDEX IDX_sscOrderItem_name ON sscOrderItem (name);

ALTER TABLE sscDiscount ADD CONSTRAINT FK_discount_orderitem FOREIGN KEY (
	orderItemID
  ) REFERENCES sscOrderItem (
	orderItemID
  ) ON DELETE SET NULL;

CREATE TABLE sscOrderItemAttribute (
  orderItemAttributeID INT IDENTITY NOT NULL,
  orderItemID INT NOT NULL DEFAULT '0',
  total DECIMAL(10,4) DEFAULT NULL,
  weight DECIMAL(10,4) DEFAULT NULL,
  costTotal DECIMAL(10,4) DEFAULT NULL,
  attributeValue TEXT NOT NULL,
  attributeID INT NOT NULL DEFAULT '0',
  attributeCode VARCHAR(100) NOT NULL DEFAULT '',
  attributeName VARCHAR(255) DEFAULT NULL,
  attributeUnitCost DECIMAL(10,4) DEFAULT NULL,
  attributeUnitPrice DECIMAL(10,4) DEFAULT NULL,
  attributeAltPrice DECIMAL(10,4) DEFAULT NULL,
  attributeWeight DECIMAL(10,4) DEFAULT NULL,
  attributeImage VARCHAR(255) DEFAULT NULL,
  optionID INT DEFAULT NULL,
  optionCode VARCHAR(100) DEFAULT NULL,
  optionName VARCHAR(255) DEFAULT NULL,
  optionUnitCost DECIMAL(10,4) DEFAULT NULL,
  optionUnitPrice DECIMAL(10,4) DEFAULT NULL,
  optionAltPrice DECIMAL(10,4) DEFAULT NULL,
  optionWeight DECIMAL(10,4) DEFAULT NULL,
  optionImage VARCHAR(255) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderItemAttributeBean',
  CONSTRAINT PK_orderItemAttributeID PRIMARY KEY CLUSTERED 
  ( orderItemAttributeID ) ON [PRIMARY],
  CONSTRAINT FK_orderItem_orderItemAttribute FOREIGN KEY (
	orderItemID
  ) REFERENCES sscOrderItem (
	orderItemID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX IDX_sscOrderItemAttribute_code ON sscOrderItemAttribute (attributeCode);
CREATE INDEX IDX_sscOrderItemAttribute_name ON sscOrderItemAttribute (attributeName);
CREATE INDEX IDX_sscOrderItemAttribute_optionCode ON sscOrderItemAttribute (optionCode);
CREATE INDEX IDX_sscOrderItemAttribute_optionName ON sscOrderItemAttribute (optionName);

CREATE TABLE sscCountry (
  countryID INT IDENTITY NOT NULL,
  name VARCHAR(255) NOT NULL,
  code VARCHAR(8) NULL,
  countryOrder INT NOT NULL,
  allowOtherStates BIT NOT NULL DEFAULT 1,
  isActive BIT NOT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.CountryBean',
  CONSTRAINT PK_countryID PRIMARY KEY CLUSTERED 
  ( countryID ) ON [PRIMARY],
  CONSTRAINT AK_countryCode UNIQUE NONCLUSTERED 
  ( code ) ON [PRIMARY]
);

CREATE INDEX IDX_sscCountry_name ON sscCountry (name);
CREATE INDEX IDX_sscCountry_order ON sscCountry (countryOrder);

CREATE TABLE sscState (
  stateID INT IDENTITY NOT NULL,
  countryID INT NOT NULL,
  name VARCHAR(255) NOT NULL,
  code VARCHAR(16) NULL,
  stateOrder INT NOT NULL,
  isActive BIT NOT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.StateBean',
  CONSTRAINT PK_stateID PRIMARY KEY CLUSTERED 
  ( stateID ) ON [PRIMARY],
  CONSTRAINT AK_stateCode UNIQUE NONCLUSTERED 
  ( code, countryID ) ON [PRIMARY],
  CONSTRAINT FK_country_state FOREIGN KEY (
	countryID
  ) REFERENCES sscCountry (
	countryID
  ) ON DELETE CASCADE  ON UPDATE CASCADE
);

CREATE INDEX IDX_sscState_name ON sscState (name);
CREATE INDEX IDX_sscState_order ON sscState (stateOrder);


CREATE TABLE sscOrderDiscount (  
  orderDiscountID INT IDENTITY NOT NULL,
  orderID INT NOT NULL DEFAULT '0',
  orderItemID INT NULL,
  total DECIMAL(10,4) DEFAULT NULL,
  discountID INT NOT NULL DEFAULT '0',
  code VARCHAR(100) NOT NULL DEFAULT '',
  name VARCHAR(255) DEFAULT NULL,
  usersAppliesType VARCHAR(32) DEFAULT NULL,
  usersAppliesValue VARCHAR(255) DEFAULT NULL,
  itemsAppliesType VARCHAR(32) DEFAULT NULL,
  itemsAppliesValue VARCHAR(255) DEFAULT NULL,
  cartItemsType VARCHAR(32) DEFAULT NULL,
  cartItemsValue VARCHAR(255) DEFAULT NULL,
  rewardType VARCHAR(32) DEFAULT NULL,
  reward DECIMAL(10,4) NOT NULL DEFAULT '0',
  couponCode VARCHAR(100) DEFAULT NULL,
  floor DECIMAL(10,4) DEFAULT NULL,
  ceiling DECIMAL(10,4) DEFAULT NULL,
  extra1 VARCHAR(255) DEFAULT NULL,
  extra2 VARCHAR(255) DEFAULT NULL,
  extra3 VARCHAR(255) DEFAULT NULL,
  class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderDiscountBean',
  CONSTRAINT PK_orderDiscountID PRIMARY KEY CLUSTERED 
  ( orderDiscountID ) ON [PRIMARY],
  CONSTRAINT FK_order_orderDiscount FOREIGN KEY (
	orderID
  ) REFERENCES sscOrder (
	orderID
  ) ON DELETE CASCADE ON UPDATE CASCADE
);

CREATE INDEX IDX_sscOrderDiscount_total ON sscOrderDiscount (total);
CREATE INDEX IDX_sscOrderDiscount_code ON sscOrderDiscount (code);
CREATE INDEX IDX_sscOrderDiscount_name ON sscOrderDiscount (name);

CREATE TABLE sscOrderSetting (
	orderSettingID INT IDENTITY NOT NULL,
	orderID INT DEFAULT NULL,
	code VARCHAR(100) NOT NULL DEFAULT '',
	name VARCHAR(255) DEFAULT NULL,
	description VARCHAR(255) DEFAULT NULL,
	type VARCHAR(100) DEFAULT NULL,
    valueType VARCHAR(16) NOT NULL,
    integerValue INT DEFAULT NULL,
	smallValue VARCHAR(64) DEFAULT NULL,
	mediumValue VARCHAR(255) DEFAULT NULL,
	textValue TEXT,
	class VARCHAR(255) DEFAULT 'com.softslate.commerce.businessobjects.order.OrderSettingBean',
    CONSTRAINT PK_orderSettingID PRIMARY KEY CLUSTERED 
    ( orderSettingID ) ON [PRIMARY],
    CONSTRAINT AK_settingCode UNIQUE NONCLUSTERED 
    ( orderID, code ) ON [PRIMARY],
    CONSTRAINT FK_orderSetting_order FOREIGN KEY (
	  orderID
    ) REFERENCES sscOrder (
	  orderID
    ) ON DELETE CASCADE ON UPDATE CASCADE
);

